/*
 * derived from mythgame
*/
#include <qapplication.h>
#include <qregexp.h>
#include <stdlib.h>
#include <iostream>
using namespace std;


#include <mythtv/mythcontext.h>
#include <mythtv/mythwidgets.h>
#include <mythtv/lcddevice.h>
#include <mythtv/xmlparse.h>

#include "Modulesettings.h"
#include "ModuleWindow.h"
#include "ViewRecipeWindow.h"
#include "recipe_filter.h"


#define RECIPE  3
#define CUISINE 2
#define COURSE  1

bool hasFocus = true;

/*
 *
 *
*/
ModuleWindow::ModuleWindow(MythMainWindow *parent, MSqlQuery *ldb, 
	    QString window_name, QString theme_filename, const char *name)
           : MythThemedDialog(parent, window_name, theme_filename, name)
{
    QString str;

    //  A few internal variable defaults
    if (ldb) {
        db = ldb;
    } else
        db = NULL;

    player_process = NULL;
    
    cout << "ModuleWindow::ModuleWindow() .." << endl;

    // setup volume for music
    //
    volume_control = NULL;
    volume_control = new VolumeControl(true);
    if (volume_control) {
        cout << tr("volume control is available...")
             << endl;
    } else {
        cout << tr("volume control is not available...")
             << endl;
    }
    showVolume(true);

    // setup the ambiant music using mpg123 or compatible player
    //
    // if disable if FM
    // check if FM is available
    // 
    str = gContext->GetSetting("RecipeMusicEnableMusic","-1");
    if (str=="-1") {
        gContext->SaveSetting("RecipeMusicEnableMusic","1");
    }
    str = gContext->GetSetting("RecipeMusicProgram", "-1");
    if (str=="-1") {
        gContext->SaveSetting("RecipeMusicProgram","mpg123");
    }
    
    if ( gContext->GetSetting("RecipeMusicEnableMusic","-1") == "1" ) {
        if ( gContext->GetSetting("RecipeMusicDisableIfFM") != "1" ) {
            player_process = new QProcess(this, "mpg123 process");
            if (player_process) {
                player_process->setWorkingDirectory( 
                                   QDir( PREFIX "/share/mythtv/mythRecipe/" ) );
                player_output_stream = new QTextStream(
                                                  player_output, IO_ReadWrite );
                if (player_output_stream) {
                    // first kill all RecipeMusicProgram...
                    player_process->addArgument( "killall" );
                    player_process->addArgument( 
                                  gContext->GetSetting("RecipeMusicProgram") );
                    if (player_process->start()) {
                        cout << tr("all kill that kill well....")
                             << endl;
                    } else 
                        cout << tr("unable to kill program....")
                             << endl;
                    
                    player_process->clearArguments();
		    
                    str = gContext->GetSetting("RecipeMusicProgram");
                    cout << tr("music program is (%1)").arg(str) << endl;
                    player_process->addArgument( str );
                    player_process->addArgument( "-C" );
                    player_process->addArgument( "-v" );
                    str = gContext->GetSetting("RecipeMusicURLList");
                    cout << tr("url to play is (%1)").arg(str) << endl;
                    player_process->addArgument( str );
                    player_process->setWorkingDirectory(
                              QDir(PREFIX "/share/mythtv/mythRecipe/"));

                    connect( player_process, SIGNAL(readyReadStdout()),
                              this, SLOT(readFromPlayerStdout()) );
                    connect( player_process, SIGNAL(processExited()),
                              this, SLOT(player_process_done()) );

                    player_output="";
                    		    
                    if (player_process->start()) {
                        cout << tr("enjoy music....")
                             << endl;
                    } else 
                        cout << tr("unable to start your music program....\n"
                                   "music will not be available ...")
                             << endl;
                } else 
                    cout << tr("unable to get output stream music program..")
                         << endl
                         << tr("something went terribly wrong...out of memory?.")
                         << endl;
            } else 
                cout << tr("unable to start ambient music program..")
                     << endl
                     << tr("something went terribly wrong.... out of memory...")
                     << endl;
        } else 
            cout << tr("FM available using it...\n"
                       "music will not be available ...")
                 << endl;
    } else 
        cout << tr("Music player not available by request...")
             << endl;

    categorylist=NULL;
    recipelist=NULL;

    recipe_tree_root=NULL;
    recipe_tree_data=NULL;

    recipeshowfavourites = FALSE;

    widget_focus=0;            // start focus on tree...
    
    // Through the magic of themes, our "GUI" already exists we just need to 
    // wire up it

    recipe_tree_root = new GenericTree("Choose a Course", 0, false);
    recipe_tree_data = recipe_tree_root->addNode(tr("All Recipes"), 0, false);

    theme=NULL;

    wireUpTheme();


    if (!buildFocusList()) {
        cout << tr("unable to build a focus list...") << endl;
    } else
        assignFirstFocus();

    // displayFilterText();
    updateStatus(0);
    
    if (recipe_tree_list) {
        recipe_tree_list->enter();
    } else {
        if (recipelist) {
             cout << tr("begin filling up recipelist...") << endl;
            fillRecipeList();
             cout << tr("filled up recipelist...") << endl;
            // nextPrevWidgetFocus(true);
        }
    }
    
    setContext( 1 );
}


/*
 *
 *
*/
ModuleWindow::~ModuleWindow(void) {

    cout << "ModuleWindow::~ModuleWindow(void)... closing..." << endl; 

    if (player_process) {
        QString str=0;
        if (gContext->GetSetting("RecipeMusicLeaveOpenWhenExit")=="1" ) {
            cout << tr("not terminating.. enjoy more music...") 
                 << endl;
        } else
            player_process->tryTerminate();
    }
}



    
/*
 * Return the type of node from the recipe tree
*/
int ModuleWindow::nodeType(GenericTree *node) {
    if (node) {
        GenericTree *p = node->getParent();
        if (p) {
            if (p->getParent()) {
                if (p->getParent()->getParent()) {
                   return RECIPE;
                } else
                   return CUISINE;
            } else 
               return COURSE;
	}
    }
    return COURSE;
}



/*
 * handle keys/button sent to us...
*/
void ModuleWindow::keyPressEvent(QKeyEvent *e) {
    bool handled = false;
    QStringList actions;
    gContext->GetMainWindow()->TranslateKeyPress("Recipe", e, actions);

    // cout << "ModuleWindow::keyPressEvent...\n" << endl;

    for (unsigned int i = 0; i < actions.size() && !handled; i++) {
        QString action = actions[i];
        handled = true;

	// cout <<"\t\t"<<action.data() << endl;

        if (action == "SELECT") {
	    enterPressed();
	}
        else if (action == "MENU" ) 
            edit();
        // else if ( action == "INFO") 
            // viewRecipe();
        else if (action == "UP")
	    cursorUp();
        else if (action == "DOWN")
	    cursorDown();
        else if (action == "LEFT")
            goLeft();
        else if (action == "RIGHT")
            goRight();
        else if (action == "PAGEUP")
            if (!recipe_tree_list->pageUp())
            {
                cerr << " Error from recipe_tree_list->pageUp" << endl;
            }
        else if (action == "PAGEDOWN")
            if(!recipe_tree_list->pageDown())
            {
		cerr << " Error from recipe_tree_list->pageDown" << endl;
            }

        else if (action == "TOGGLEFAV")
            toggleFavourite();
        else if (action == "MUTE") { 
            if (mute_button)
                mute_button->push();
	    if (muted) {
                if (volume_control)
	           volume_control->SetMute(FALSE);
	    } else {
                if (volume_control)
	           volume_control->SetMute(TRUE);
	    }
        }
        else if (action == "VOLUMEDOWN") {
            if (volume_control)
                volume_control->AdjustCurrentVolume(-1);
        } else if (action == "VOLUMEUP") {
            if (volume_control)
                volume_control->AdjustCurrentVolume(1);
        } if (action.toInt()>0) {
            int button = action.toInt();
            
            if ( button<4 ) {
                if (tab_button[button-1])
                    tab_button[button-1]->push();
            }
            
        } else
            handled = false;
    }

    if (!handled)
        MythThemedDialog::keyPressEvent(e);
    
    updateForeground();
}


/*
 *
 * 
*/
void ModuleWindow::setNewContext( int key) {

    if (key<4) {
        setContext( key );
        
        nextPrevWidgetFocus(true);
    } else {
    }
}



/*
 *
 * 
*/
void ModuleWindow::fillRecipeList() {
    cerr << "ModuleWindow::fillRecipeList: ..." << endl;
    QString filter="";

    if (recipelist) {
        if (db) {
            QString favfilter;
            filter = ("SELECT DISTINCT name, favourite FROM "
                     "recipes, category_entries, category, ingredients"
                     " WHERE recipe.recipeid=category_entries.recipeid"
                     " AND category_entries.category_id=categories.category_id"
		     " AND recipes_recipe_id=ungredients.recipe_id ");
            filter = ("SELECT DISTINCT recipes.name, favourite FROM recipes, "
                      "category_entries, categories, ingredients WHERE "
                      "recipes.recipe_id=category_entries.recipe_id AND "
                      "category_entries.category_id=categories.category_id AND "
                      "recipes.recipe_id=ingredients.recipe_id ");
/*
            if (recipeshowfavourites)
                favfilter=" AND favourite=1 ";
	    if (!(recipenamefilter.isEmpty() || recipenamefilter=="all")) {
		favfilter=(recipeshowfavourites ? " AND favourite=1 ":"");
                filter = filter + "SELECT DISTINCT name, favourite \
			  FROM recipes WHERE name LIKE \'\%" + 
		            + recipenamefilter + "\%\' " + favfilter +
		  " ORDER BY name;";
            } else {
                if (recipeshowfavourites)
                    favfilter = " WHERE favourite=1 ";
                filter= filter + "SELECT DISTINCT name, favourite FROM recipes " +
			favfilter + " ORDER BY name;";
     	    }
*/
            if (recipeshowfavourites)
                filter += " AND recipes.favourite=1 ";

            if (!(recipenamefilter.isEmpty() || recipenamefilter=="all" ||
                                                recipenamefilter.isNull() ) ) 
                filter += QString("AND recipes.name LIKE \'\%%1\%\'")
                                 .arg(recipenamefilter);

            if (!(recipecoursefilter.isEmpty() || recipecoursefilter=="all" ||
                                                 recipecoursefilter.isNull())) 
                filter += QString("AND categories.name LIKE \'\%%1\%\'")
                                  .arg(recipecoursefilter);

            if (!(recipecuisinefilter.isEmpty()||recipecuisinefilter=="all" ||
                                                recipecuisinefilter.isNull())) 
                filter += QString("AND categories.name LIKE \'\%%1\%\'  ")
                                  .arg(recipecuisinefilter);

            if (!(recipeingredientfilter.isEmpty()||recipeingredientfilter=="all"
                                           ||recipeingredientfilter.isNull())) 
                filter += QString("AND ingredients.name LIKE \'\%%1\%\'  ")
                                  .arg(recipeingredientfilter);
                                  
            filter += QString("ORDER By recipes.name");
            
            cerr << "ModuleWindow::fillRecipeList: ..." 
                 << filter
                 << endl;

            db->prepare(filter);
            db->exec();
 
            recipelist->Reset();
            while (db->next()) {
                UIListBtnTypeItem *item = new UIListBtnTypeItem(recipelist, 
                                                      db->value(0).toString(),
                                          new QPixmap("favourite.pnm"), true);
                if (db->value(1).toBool()) { // if favorite
                    // item->setPixmap(new QPixmap("favourite.pnm"));
                   item->setCheckable(true);
                   item->setChecked(UIListBtnTypeItem::FullChecked);
                } else {
                   item->setCheckable(false);
                }
            }
        } else
            cerr << "ModuleWindow::fillRecipeList: ... database not ready "
                 << endl;
    } else
        cerr << "ModuleWindow::fillRecipeList: ... don't have a recipelist "
             << endl;
}


/*
 *
 * 
*/
void ModuleWindow::fillRecipeTree(QString /* filter */) {
    cerr << "ModuleWindow::fillRecipeTree: ..." << endl;
    if (recipe_tree_list) {
        recipe_tree_list->showWholeTree(true);
        recipe_tree_list->colorSelectables(true);
	
        if (db) {
            QString thequery= "SELECT DISTINCT category_id, name "
		                      "FROM categories "
				              "WHERE parent=0 "
					          "ORDER BY name;";	    
            db->prepare(thequery);
            db->exec();
            while (db->next()) {
                recipe_tree_data=recipe_tree_root->addNode(
                                           db->value(1).toString(), 0, false);
                QString crsquery;
                crsquery.sprintf( "SELECT DISTINCT category_id, name \
				        FROM categories \
					     WHERE parent=%i \
					          ORDER BY name;", db->value(0).toInt());
                MSqlQuery cquery(MSqlQuery::InitCon());
                cquery.prepare(crsquery);
                cquery.exec();
                while (cquery.next()) {
                    recipe_tree_data->addNode(
                                          cquery.value(1).toString(),0,false);
                }
            }
        } else
            cerr <<"ModuleWindow::fillRecipeTree:...database not ready "<<endl;

        recipe_tree_list->assignTreeData(recipe_tree_root);

        connect(recipe_tree_list, SIGNAL(nodeSelected(int, IntVector*)), 
                    this, SLOT(handleTreeListSelection(int, IntVector*)));
        connect(recipe_tree_list, SIGNAL(nodeEntered(int, IntVector*)), 
                    this, SLOT(handleTreeListEntry(int, IntVector*)));
    } else
        cerr <<"ModuleWindow::fillRecipeTree:...don't have a recipetreelist"<<endl;
}



/*
 *
 *
*/
void ModuleWindow::button_enter_focus() {

    nextPrevWidgetFocus(true);
        
    cout << "ModuleWindow::button_enter_focus...\n" << endl;
}



/*
 *
 *
*/
void ModuleWindow::ingredients_list_enter_focus() {

    cout << "ModuleWindow::ingredients_list_enter_focus...\n" << endl;
    
    if (getContext()!=2) {
        nextPrevWidgetFocus(true);
        nextPrevWidgetFocus(true);
    } else {
        if (ingredientslist) {
            ingredientslist->SetActive(true);
            m_InColumn=4;
            if (info0_image) {
                info0_image->SetImage( "active-box-0.png" );
                info0_image->LoadImage();
            }
        }
    }
}
	


/*
 *
 *
*/
void ModuleWindow::recipes_tree_enter_focus() {

    if (info3_image) {
        info3_image->SetImage( "background.png" );
        info3_image->LoadImage();
    }
    cout << "ModuleWindow::recipes_tree_enter_focus...\n" << endl;
    m_InColumn=3;
}
	


/*
 *
 *
*/
void ModuleWindow::directions_list_enter_focus() {

    cout << "ModuleWindow::directions_list_enter_focus...\n" << endl;
    if (getContext()!=3) {
        nextPrevWidgetFocus(true);
    } else {
        m_InColumn=5;
        if (info2_image) {
            info2_image->SetImage( "active-box-2.png" );
            info2_image->LoadImage();
        }
    }
}
	


/*
 *
 *
*/
void ModuleWindow::recipelist_list_enter_focus() {

    if (recipelist) {
       recipelist->SetActive(true);
       m_InColumn=2;
        if (info3_image) {
            info3_image->SetImage( "background.png" );
            info3_image->LoadImage();
        }
    }
    cout << "ModuleWindow::recipelist_list_enter_focus...\n" << endl;
}	
	


/*
 *
 *
*/
void ModuleWindow::filter_list_enter_focus() {

    if (filter_text) {
	// recipelist->SetActive(true);
        m_InColumn=0;
        if (info3_image) {
            info3_image->SetImage( "background.png" );
            info3_image->LoadImage();
        }
        if (info4_image) {
            info4_image->SetImage( "active-box-3.png" );
            info4_image->LoadImage();
        }
    }
    cout << "ModuleWindow::filter_list_enter_focus...\n" << endl;
}



/*
 *
 *
*/
void ModuleWindow::button_loose_focus() {
        
    cout << "ModuleWindow::button_loose_focus...\n" << endl;
}


/*
 *
 *
*/
void ModuleWindow::ingredients_list_loose_focus() {

    if (ingredientslist) {
        ingredientslist->SetActive(false);
        if (info0_image) {
    	    info0_image->SetImage( "inactive-box-0.png" );
    	    info0_image->LoadImage();
        }
    }
    cout << "ModuleWindow::ingredients_list_loose_focus...\n" << endl;
    // updateForeground();
}
	


/*
 *
 *
*/
void ModuleWindow::recipes_tree_loose_focus() {

    if (info3_image) {
        info3_image->SetImage( "" );
        info3_image->LoadImage();
    }
    cout << "ModuleWindow::recipes_tree_loose_focus...\n" << endl;
    // m_InColumn=0;
}
	


/*
 *
 *
*/
void ModuleWindow::directions_list_loose_focus() {

    if (info2_image) {
        info2_image->SetImage( "inactive-box-2.png" );
        info2_image->LoadImage();
    }
    cout << "ModuleWindow::directions_list_loose_focus...\n" << endl;
}
	


/*
 *
 *
*/
void ModuleWindow::recipelist_list_loose_focus() {

    if (recipelist) {
        recipelist->SetActive(false);
        if (info3_image) {
            info3_image->SetImage( "" );
            info3_image->LoadImage();
        }
    }
    cout << "ModuleWindow::recipelist_list_loose_focus...\n" << endl;
    //m_InColumn=0;
}
	


/*
 *
 *
*/
void ModuleWindow::filter_list_loose_focus() {

    if (info4_image) {
        info4_image->SetImage( "" );
        info4_image->LoadImage();
    }
    cout << "ModuleWindow::recipes_tree_loose_focus...\n" << endl;
}

	


/*
 *
 *
*/
void ModuleWindow::categoryListChanged( UIListBtnTypeItem *item) {

    //cout << "ModuleWindow::categoryListChanged()...";
    if (item) {
        // cout << " got "<< item->text().data();
    }
    // out << endl;
}



/*
 *
 *
*/
void ModuleWindow::recipeListChanged( UIListBtnTypeItem *item) {
    QString str;
    // cout << "ModuleWindow::recipeListChanged()... ";
    if (item) {
        // cout << " got "<< item->text().data();
        displayRecipe(item->text());
        if ( current_recipe_text ) {
            str = QString("rec %1/%2")
                          .arg(recipelist->GetItemPos(item)+1) 
                          .arg(recipelist->GetCount());
            current_recipe_text->SetText( str );
        }
    }
    // cout << endl;
}



/*
 *
 *
*/
void ModuleWindow::ingredientsListChanged( UIListBtnTypeItem *item) {
    // cout << "ModuleWindow::ingredientListChanged()... ";

    if (ingredientslist) 
        if (item)         
            ingredient_text->SetText( item->text() );
}



/*
 *
 *
*/
void ModuleWindow::goRight() {
    QString str="";
    cout << tr("ModuleWindow::goRight()... column(%1)")
              .arg( m_InColumn )
         << endl;

    switch (m_InColumn) {
        case 3 :
                if (recipe_tree_list) {
                    GenericTree *node = recipe_tree_list->getCurrentNode();
                    switch (nodeType(node)) {
                        case CUISINE:
                                      if (node->childCount()<1) {
                                          addRecipeToList(node);
                                      }
                                      recipe_tree_list->pushDown();
                                      break;
                        case RECIPE:
                                      // nextPrevWidgetFocus(true);
                                      switch (getContext()) {
                                          case 2:
                                                  if (ingredientslist) 
                                                      ingredientslist->takeFocus();
                                                  break;
                                          case 3:
                                                  if (directions_text) 
                                                      directions_text->takeFocus();
                                                  break;
                                          case 1:
                                                  break;
                                          default:
                                                   break;
                                      }
                                      break;
                        case COURSE:
                                      if ((node->getString())=="All Recipes")
                                          if ( (node->childCount()<1) ) 
                                              node->addNode("All Recipes",0,false);
                                      recipe_tree_list->pushDown();
		                break;
                    }
                }
                break;
        case 4 :
                 nextPrevWidgetFocus(true);
        case 5:
        case 2 :
                 nextPrevWidgetFocus(true);
                 break;
    }
}


/*
 *
 *
*/
void ModuleWindow::goLeft() {
    // cout <<"ModuleWindow::goLeft()... column (" << m_InColumn <<") " << endl;

                if (recipe_tree_list) {
                    recipe_tree_list->popUp();
                    GenericTree *node = recipe_tree_list->getCurrentNode();
                    switch (nodeType(node)) {
                        case COURSE:
                                      break;
                        case CUISINE:
                                      break;
                        case RECIPE:
                                      break;
                    }
                } else {
                    // updateRecipelistTreeTitle( recipe_tree_list->getCurrentNode());
                    nextPrevWidgetFocus(false);
                }
    // updateForeground();
}


/*
 *
 *
*/
void ModuleWindow::toggleFavourite() {

    if (recipe_tree_list) {
        recipe_tree_list->select();
    } else {
        if (recipelist) {
            UIListBtnTypeItem *item = recipelist->GetItemCurrent();
            int favourite=0;
		    
            if (!item)
                return;
                
            if (item->checkable()) {
                item->setCheckable(false);
                favourite=0;
                // item-ingredientslist>setChecked(UIListBtnTypeItem::FullChecked);
            } else {
                favourite=1;
                item->setCheckable(true);
                item->setChecked(UIListBtnTypeItem::FullChecked);
            }
            
	        if (db) {
                QString query = QString("UPDATE recipes SET favourite=%1"
                                        " WHERE name='%2'")
                                        .arg(favourite)
                                        .arg(item->text());
                
			    cout << query <<endl;
                        // QSqlQuery query = db->exec(update);
                        db->prepare(query);
                        db->exec();
	        }

        } 
    }
}



/*
 *
 *
*/
void ModuleWindow::next() {

}


/*
 *
 *
*/
void ModuleWindow::previous() {

}


/*
 *
 *
*/
void ModuleWindow::seeknext() {

}


/*
 *
 *
*/
void ModuleWindow::seekprev() {

}


/*
 *
 *
*/
void ModuleWindow::stop() {

}


/*
 *
 *
*/
void ModuleWindow::record() {

}


/*
 *
 *
*/
void ModuleWindow::handleTreeListSelection(int, IntVector*) {
    GenericTree *node = recipe_tree_list->getCurrentNode();
    // cout << "ModuleWindow::handleTreeListSelection().." << endl; 
    nextPrevWidgetFocus(true);
    switch (nodeType(node)) {
        case CUISINE:
                      break;
        case RECIPE:
                      break;
        case COURSE:
                      break;
    }


}


/*
 *
 *
*/
void ModuleWindow::handleTreeListEntry(int, IntVector*) {
    if (recipe_tree_list) {
        GenericTree *node = recipe_tree_list->getCurrentNode();
        // cout << "ModuleWindow::handleTreeListEntry().." << endl;
        
        switch (nodeType(node)) {
            case CUISINE:
        		break;
            case RECIPE:
        		displayRecipe(node->getString());
        		break;
            case COURSE:
        		break;
        }
    }
}


/*
 *
 *
*/
void ModuleWindow::displayRecipe( QString recipename) {
    QString thequery=0;
    QSqlQuery query;
    QString ingredients=0;
    QString str=0;
    QString ing_str=0;
    QStringList strlst;

    static QString oldrecipename=0;

    cout << "\nModuleWindow::displayRecipe()..." << endl; 
    
    if (oldrecipename==recipename)
        return;

    oldrecipename=recipename;

    int recipeid=0;
    int authorid=0;

    if (db) {
        thequery = ("SELECT DISTINCT recipe_id, "        // 0
                                     "name, "            // 1
                                     "yield_qty, "       // 2
                                     "yield_units, "     // 3
                                     "source, "          // 4
                                     "author_user_id, "  // 5
                                     "comment, "         // 6
                                     "extra_comment, "   // 7
                                     "image_name, "      // 8
                                     "recipe_image "     // 9
                                     "FROM recipes WHERE "
                         "name=\"" + recipename + "\" ;" );
        db->prepare(thequery);
        db->exec();
        if (!db->next()) {
            cout << tr("recipe not found ...")
                 << endl;
            return;
        }
        else cout << tr("found recipe") << recipename << endl;

        recipeid = (db->value(0).toInt());
        authorid = (db->value(5).toInt());

        if (recipeid<1)
            return;

        if (serving_text) 
            serving_text->SetText( QString("%1 %2 ")
                          .arg( (db->value(2).toString()=="0")  
                                           ? "" : (db->value(2).toString()) )
                          .arg(db->value(3).toString() )
                          );
                      
        if (source_text) 
            source_text->SetText( db->value(4).toString() );

        if (comment_text)  {
            comment_text->SetText( db->value(6).toString() );
            if (extra_comment_text) 
                extra_comment_text->SetText( db->value(7).toString() );
        }

        if (recipe_image) {
            QString img_name = db->value(8).toByteArray();
            cout << "Debug: recipe_image = "<< db->value(8).toString() << endl;

            if (img_name=="noimage.jpg") {
                recipe_image->SetImage("noimage.jpg");
            } else {
                QString image_name = QString(PREFIX 
                                           "/share/mythtv/themes/default/recipes/%1") 
                                          .arg(img_name);
                cout << "Debug: QString image_name =" << image_name << endl;

                QFile file(image_name);
                if (file.exists()) {
                    QString theme_image_name = QString("/recipes/%1").arg(img_name);
                    cout << "Debug: Image file.exists() " << endl;
                    recipe_image->SetImage(theme_image_name);  // this is looking under the Theme selected, not default
//                    recipe_image->SetImage(img_name);  // this is looking under the Theme selected, not default
                } else {
                    cout << "Debug: Reading image from DB size="<< sizeof(db->value(9).toByteArray()) << endl;
                    QByteArray img_data = db->value(9).toByteArray();
                    
                    cout << "Image Size=" << sizeof(img_data) << endl;
                    if (!img_data.isNull()) {
                        cout << "Debug: Image from DB is Not NULL.  Displaying " << endl;
                        QPixmap img( img_data );
                        recipe_image->SetImage(img);
                    } 
                    else {
                        cout << "Debug: DB image Null " << endl;
                    }
                }

            }
            recipe_image->LoadImage();
        }
	

        if (author_text) {
            author_text->SetText("");
            thequery.sprintf("SELECT DISTINCT user_name FROM users "
                             "WHERE user_id=%i;", db->value(5).toInt());
        
            db->prepare(thequery);
            db->exec();
            
            if (db->next()) {
	            author_text->SetText( db->value(0).toString() );
            } else 
                author_text->SetText( "unKnown user" );
	    }

        if (directions_text) {
            directions_text->SetText("");
            thequery.sprintf("SELECT step_num, step_text " 
			 "FROM instructions WHERE recipe_id=%i ;", recipeid );

            db->prepare(thequery);
            db->exec();

            directions_cnt=1;
            directions_top=0;
            cur_directions.clear();
            
            while (db->next()) {
	            cur_directions.append( db->value(1).toString() );
            }
            
            displayDirections(directions_top);
        }
	
          
        thequery.sprintf("SELECT ingredient_id, recipe_id, ingredients.name,"
                         "display_order, ingredient_type, sub_recipe_id, qty," 
                         "ingredients.unit_id, units.name, units.abbreviation "
                         "FROM ingredients, units "
                         "WHERE recipe_id=%i AND "
                         "ingredients.unit_id=units.unit_id;", recipeid );
        db->prepare(thequery);
        db->exec();
	

        if (ingredientslist) {
            ingredientslist->Reset();
            while (db->next()) {
                str = (db->value(2).toString());
                // ing_str = str.replace( QRegExp("\\d*||\[]"), "+");
                ing_str = str.replace(QRegExp("\\d*||transp.gif||\[]\[]"),"");
                
                switch (db->value(4).toInt()) {
                case 0 :
                case 1 :
                        ingredients.sprintf("%3.2f %s %s", 
                                            db->value(6).toString().toFloat(),
                                            db->value(9).toString().data(), 
                                            ing_str.data() );
                        break;
                case 2 :
                        ingredients.sprintf("%s", ing_str.data() );
                        ingredients = ing_str;
                        break;
                case 3 :
                        ingredients.sprintf("%s", ing_str.data() );
                        ingredients = ing_str;
                        break;
                case 4 :
                        ingredients.sprintf("%s", ing_str.data() );
                        ingredients = ing_str;
                        break;
	            }
                new UIListBtnTypeItem(ingredientslist, ingredients.data());
            }
        }
    }
}



/*
 *
 *
*/
void ModuleWindow::displayFilterText() {
    QString fs;

    recipecoursefilter    =gContext->GetSetting("RecipeCourseFilter").lower();
    recipecuisinefilter   =gContext->GetSetting("RecipeCuisineFilter").lower();
    recipeingredientfilter=gContext->GetSetting("RecipeIngredientFilter").lower();
    recipenamefilter      =gContext->GetSetting("RecipeNameFilter").lower();
    recipeshowfavourites  =(gContext->GetSetting("RecipeShowFavourites")=="1");

    fs = fs + "Name: " + recipenamefilter.left(15) + "\n";
    fs = fs + "Course: " + recipecoursefilter.left(15) + "\n";
    fs = fs + "Cuisine: " + recipecuisinefilter.left(15) + "\n";
    fs = fs + "Ingredient: " + recipeingredientfilter.left(15) + "\n";
    fs = fs + "Show Favourite? "+(recipeshowfavourites ? "Yes":"No ") + "\n";

    filter_text->SetText(fs);
}



/*
 *
 *
*/
void ModuleWindow::displayDirections(int /* topdirections */) {
    QString directionstext="";
    QStringList::Iterator it;
    int cnt=-1;

    // cout << "ModuleWindow::displayDirections()..." << endl; 

    if (!directions_text)
        return;

    directions_text->SetText("");

    if (directions_top<0)
        directions_top=0;

    for (it=cur_directions.begin();it!=cur_directions.end();++it){
	cnt++;
	if (cnt>directions_top) {
            directionstext.sprintf("%s%i. %s\n",
			    directionstext.data(),cnt,(*it).data());
	}
    }

    if (directionstext.isEmpty()) {
        directionstext.sprintf("%s%i. %s\n",
			directionstext.data(),cnt,(*(--it)).data());
	directions_top=cnt-1;
    }

    directions_text->SetText(directionstext);
}


/*
 *
 *
*/
void ModuleWindow::addRecipeToList(GenericTree *node) {
    QString thequery;
    QSqlQuery query;
    QString parent;
    QString child;

    cout << "ModuleWindow::addRecipeToList()..\n";

    if (nodeType(node)==CUISINE) {
        if (node->childCount()<1) {
            if (db) {
                parent=(node->getParent())->getString();
                child=node->getString();
                if ( (child=="All Recipes") ) {
                    thequery = QString("SELECT name "
                                       "FROM recipes ORDER BY name");
                    db->prepare(thequery);
                    db->exec();
                    while (db->next()) 
                         node->addNode(db->value(0).toString(), 0, false);
                } else {
                    // get parent course
                    thequery = QString( "SELECT category_id "
		                                "FROM categories "
		                                "WHERE parent=0 AND name=\"%1\"")
		                              .arg( parent );
                    db->prepare(thequery);
                    db->exec();
            
                    db->next();

                    thequery = QString("SELECT DISTINCT category_id "
				                       "FROM categories "
				                       "WHERE parent=%1 AND name=\"%2\"")
				                     .arg(db->value(0).toInt())
				                     .arg( child.data());
                    db->prepare(thequery);
                    db->exec();
            
                    if (db->next()) {

                        thequery = QString("SELECT recipes.name "
		                                   "FROM category_entries, recipes "
                                "WHERE category_entries.category_id=%1 AND "
			                 "recipes.recipe_id=category_entries.recipe_id "
 		                     "ORDER BY recipes.name ;")
 		                     .arg(db->value(0).toInt());
 		        
                        db->prepare(thequery);
                        db->exec();
                
                        while (db->next()) 
                            node->addNode(db->value(0).toString(), 0, false);
		            }
		        }
            }
	    }
    } 
}


void ModuleWindow::edit() {
    cout <<"ModuleWindow::edit..." << endl;
    MythRecipeFilterSettings rf;
    QString fs="";
        
    // rf.exec(db,true);
    rf.exec();

    updateStatus(0);
}


/*
 *
 *
*/
void ModuleWindow::viewRecipe() {
    QString recipe_name=0;
return;
    switch (m_InColumn) {
        case 3 :
                if (recipe_tree_list) {
                    GenericTree *node = recipe_tree_list->getCurrentNode();
                    // cout << "ModuleWindow::handleTreeListEntry().." << endl;
                    
                    switch (nodeType(node)) {
                        case CUISINE:
                    		break;
                        case RECIPE:
                    		recipe_name = node->getString();
                    		break;
                        case COURSE:
                    		break;
                    }
                }
                break;
        case 2 :
		if (recipelist) {
		    UIListBtnTypeItem *item = recipelist->GetItemCurrent();
                    if (item)
                       recipe_name = item->text();
                }
                break;
        case 0 :
                break;
    }

    
    if ( !(recipe_name.isEmpty() || recipe_name.isNull()) ) {

        ViewRecipeWindow vr(gContext->GetMainWindow(), 
			    db,
	                    "Recipe", 
	                    "Recipe-view-", 
                            "Recipe");
        vr.displayRecipe( recipe_name );
        vr.exec();
    }
}


/*
 *
 *
*/
void ModuleWindow::numberPressAction(int ) {

}


/*
 *
 *
*/
void ModuleWindow::cursorPgUp() {

}


/*
 *
 *
*/
void ModuleWindow::cursorUp() {
    // cout << tr("ModuleWindow::cursorUp()... column (%1)").arg(m_InColumn)
    //      << endl; 

    switch (m_InColumn) {
        case 0:
            nextPrevWidgetFocus(true);
            cursorUp();
        case 1:
            break;
        case 2:
            if (recipelist) {
                recipelist->MoveUp();
	    }
            break;
        case 3:
            if (recipe_tree_list)
                recipe_tree_list->moveUp();
            break;
        case 4:
            if (ingredientslist)
                ingredientslist->MoveUp();
            break;
        case 5:
            displayDirections(--directions_top);
            break;
        default:
            break;
    }
}



/*
 *
 *
*/
void ModuleWindow::cursorPgDown() {

}


/*
 *
 *
*/
void ModuleWindow::cursorDown() {
    // cout << tr("ModuleWindow::cursorDown()... column (%1)").arg(m_InColumn)
    //      << endl; 
    switch (m_InColumn) {
        case 0:
            nextPrevWidgetFocus(true);
	    cursorDown();
        case 1:
            break;
        case 2:
            if (recipelist) {
                recipelist->MoveDown();
	    }
            break;
        case 3:
            if (recipe_tree_list) {
                recipe_tree_list->moveDown();
	    }
            break;
        case 4:
            if (ingredientslist) {
                ingredientslist->MoveDown();
	    }
            break;
        case 5:
            displayDirections(++directions_top);
            break;
        default:
	    break;
    }
}


/*
 *
 *
*/
void ModuleWindow::enterPressed() {
    // cout << "ModuleWindow::enterPressed()... column (" 
	//      << m_InColumn << ")" << endl; 

		    /*
		    displayFilterText();

                    if (recipelist) {
                        fillRecipeList();
                    }
		    */
		    
		    /*
            if (recipe_tree_list) {
                recipe_tree_list->enter();
            } else {            
                MythRecipeFilterSettings rf;

                rf.exec();
                updateStatus(0);
            }
            */
}



/*
 *
 *
*/
void ModuleWindow::showVolume(bool /* on_or_off */) {

}


/*
 *
 *
*/
void ModuleWindow::updateStatus( QString ) {

    if (author_text)
	author_text->SetText(0);
    if (source_text)
        source_text->SetText(0);
    if (serving_text)
        serving_text->SetText(0);
    if (comment_text)
        comment_text->SetText(0);
    if (ingredient_text)
        ingredient_text->SetText(0);
    if (directions_text)
        directions_text->SetText(0);

    if (ingredientslist)
	ingredientslist->Reset();
    if (recipelist)
	recipelist->Reset();
    if (categorylist)
	categorylist->Reset();

    if (recipe_tree_list) {
        fillRecipeTree( recipenamefilter ); 
    } else {
        if (recipelist) {
            displayFilterText();
            fillRecipeList();
        }
    }
}


/*
 *
 *
*/
void ModuleWindow::buildRecipeList( bool /* all */, int /* cur_list */ ) {

}


/*
 *
 *
*/
void ModuleWindow::buildCategoryList() {

}


/*
 *
 *
*/
void ModuleWindow::updateRecipelistTreeTitle( GenericTree * ) {

}



/*
 *
 *
*/
void ModuleWindow::closeEvent(QCloseEvent *event) {
   // check if you need ambient music while you run other modules...
   //
   
   // if (need_ambient) {
   //     leave alone
   // } else {
   //     fmradio->();
   //     or 
   //     exec(mpg123 some.url)
   // }

    hide();
    event->accept();
}



/*
 *
 *
*/
void ModuleWindow::showEvent(QShowEvent *event) {
    QWidget::showEvent(event);
}



/*
 *
 *
*/
void ModuleWindow::customEvent(QCustomEvent *event) {
    QWidget::customEvent(event);
}


/*
 *
 *
*/
void ModuleWindow::wireUpTheme() {
    QString str;

    // m_InColumn=3;

    // All the other GUI elements are **optional**

    recipe_tree_list = getUIManagedTreeListType("recipetreelist");
    if (!recipe_tree_list) {
        cout << "\t\tunable to get recipetreelist ..." << endl;
    } else {
        cout << "\t\tgot recipetreelist ..." << endl;
        recipe_tree_list->allowFocus( true );
        recipe_tree_list->calculateScreenArea();
    }

    recipetreelisttitle_text = getUITextType("recipetreelisttitle");
    if (!recipetreelisttitle_text) {
        cout << "\t\tunable to get recipetreelist text ..." << endl;
    } else {
        recipetreelisttitle_text->SetJustification(Qt::AlignLeft|Qt::WordBreak);
        recipetreelisttitle_text->SetCutDown(false);
        cout << "\t\tgot recipetreelist text ..." << endl;
    }

    author_text = getUITextType("author");
    if (!author_text) {
        cout << "\t\tunable to get author text ..." << endl;
    } else {
        author_text->SetJustification(Qt::AlignLeft|Qt::WordBreak);
        author_text->SetCutDown(true);
        author_text->allowFocus( false );
    }

    source_text = getUITextType("source");
    if (!source_text) {
        cout << "\t\tunable to get source text ..." << endl;
    } else {
        source_text->SetJustification(Qt::AlignLeft|Qt::WordBreak);
        source_text->SetCutDown(true);
        source_text->allowFocus( false );
    }

    serving_text = getUITextType("serving");
    if (!serving_text) {
        cout << "\t\tunable to get serving text ..." << endl;
    } else {
        serving_text->SetJustification(Qt::AlignLeft|Qt::WordBreak);
        serving_text->SetCutDown(true);
        serving_text->allowFocus( false );
    }

    comment_text = getUITextType("comment");
    if (!comment_text) {
        cout << "\t\tunable to get comment text ..." << endl;
    } else {
        comment_text->SetJustification(Qt::AlignLeft|Qt::WordBreak);
        comment_text->SetCutDown(false);
        comment_text->allowFocus( false );
    }

    extra_comment_text = getUITextType("extra_comment");
    if (!extra_comment_text) {
        cout << "\t\tunable to get extra_comment text ..." << endl;
    } else {
        extra_comment_text->SetJustification(Qt::AlignLeft|Qt::WordBreak);
        extra_comment_text->SetCutDown(false);
        extra_comment_text->allowFocus( false );
    }

    total_recipe_text = getUITextType("total_recipe_text");
    if (!total_recipe_text) {
        cout << "\t\tunable to get total_recipe_text text ..." << endl;
    } else {
        total_recipe_text->SetJustification(Qt::AlignLeft|Qt::WordBreak);
        total_recipe_text->SetCutDown(false);
        total_recipe_text->allowFocus( false );
    }

    current_recipe_text = getUITextType("current_recipe_text");
    if (!current_recipe_text) {
        cout << "\t\tunable to get current_recipe_text text ..." << endl;
    } else {
        current_recipe_text->SetJustification(Qt::AlignLeft|Qt::WordBreak);
        current_recipe_text->SetCutDown(false);
        current_recipe_text->allowFocus( false );
    }

    recipe_image = getUIImageType("recipeimage");
    if (!recipe_image) {
        cout << "\t\tunable to get recipe image ..." << endl;
    } else {
        recipe_image->allowFocus( false );
    }

    info0_image = getUIImageType("info0");
    if (!info0_image) {
        cout << "\t\tunable to get ingredients image ..." << endl;
    } else {
        info0_image->allowFocus( false );
    }

    info1_image = getUIImageType("info1");
    if (!info1_image) {
        cout << "\t\tunable to get info image ..." << endl;
    } else {
        info1_image->allowFocus( false );
    }
    

    info2_image = getUIImageType("info2");
    if (!info2_image) {
        cout << "\t\tunable to get directions image ..." << endl;
    } else {
        info2_image->allowFocus( false );
    }

    info3_image = getUIImageType("filler");
    if (!info3_image) {
        cout << "\t\tunable to get filler image ..." << endl;
    } else {
        info3_image->allowFocus( false );
    }

    info4_image = getUIImageType("info3");
    if (!info4_image) {
        cout << "\t\tunable to get filter image ..." << endl;
    } else {
        info4_image->allowFocus( false );
    }

    directions_text = getUITextType("directions");
    if (!directions_text) {
        cout << "\t\tunable to get directions text ..." << endl;
    } else {
        directions_text->SetJustification(Qt::AlignLeft|Qt::WordBreak);
        directions_text->SetCutDown(true);
        directions_text->allowFocus(true);
        directions_text->calculateScreenArea();
    }

    ingredient_text = getUITextType("ingredient");
    if (!ingredient_text) {
        cout << "\t\tunable to get ingredient text ..." << endl;
    } else {
        ingredient_text->SetJustification(Qt::AlignLeft|Qt::WordBreak);
        ingredient_text->SetCutDown(true);
    }
    
    ingredientslist = getUIListBtnType("ingredients");
    if (!ingredientslist) {
        cout << "\t\tunable to get ingredients list ..." << endl;
    } else {
        cout << "\t\tgot ingredients list ..." << endl;
        ingredientslist->SetVisible(true);
        ingredientslist->allowFocus( true );
        ingredientslist->SetActive( false );
        ingredientslist->calculateScreenArea();
    }

    recipelist = getUIListBtnType("recipelist");
    if (!recipelist) {
        cout << "\t\tunable to get recipelist ..." << endl;
    } else {
        recipelist->SetVisible(true);
        recipelist->allowFocus( true );
        recipelist->SetActive( false );
        recipelist->calculateScreenArea();
        cout << "\t\tgot recipelist list ..." << endl;
    }

    categorylist = getUIListBtnType("categorylist");
    if (!categorylist) {
        cout << "\t\tunable to get categorylist ..." << endl;
    } else {
        categorylist->SetVisible(true);
        categorylist->allowFocus( true );
        categorylist->SetActive( false );
        categorylist->calculateScreenArea();
        cout << "\t\tgot categorylist list ..." << endl;
    }


    filter_text = getUITextType("filter");
    if (!filter_text) {
        cout << "\t\tunable to get filter text ..." << endl;
    } else {
        filter_text->SetJustification(Qt::AlignLeft|Qt::WordBreak);
        filter_text->SetCutDown(true);
        filter_text->allowFocus(false);
        filter_text->calculateScreenArea();
        filter_text->SetText("..Show All..");
        cout << "\t\tgot filter text ..." << endl;
    }

    if (recipetreelisttitle_text)
        recipetreelisttitle_text->SetText("Select a Course...");

    if (categorylist)
         connect(categorylist, SIGNAL(itemSelected( UIListBtnTypeItem *)),
                 this, SLOT(categoryListChanged( UIListBtnTypeItem *)));
    if (recipelist)
         connect(recipelist, SIGNAL(itemSelected( UIListBtnTypeItem *)),
                 this, SLOT(recipeListChanged( UIListBtnTypeItem *)));
    if (ingredientslist)
         connect(ingredientslist, SIGNAL(itemSelected( UIListBtnTypeItem *)),
                 this, SLOT(ingredientsListChanged( UIListBtnTypeItem *)));
    
    // Focus enter events...

    if (recipe_tree_list)
         connect(recipe_tree_list, SIGNAL(takingFocus()),
                 this, SLOT( recipes_tree_enter_focus() ) );
    if (ingredientslist)
         connect(ingredientslist, SIGNAL(takingFocus()),
                 this, SLOT( ingredients_list_enter_focus() ) );
    if (directions_text)
         connect(directions_text, SIGNAL(takingFocus()),
                 this, SLOT( directions_list_enter_focus() ) );
    if (recipelist)
         connect(recipelist, SIGNAL(takingFocus()),
                 this, SLOT( recipelist_list_enter_focus() ) );
    if (filter_text)
         connect(filter_text, SIGNAL(takingFocus()),
                 this, SLOT( filter_list_enter_focus() ) );

    // Focus loose events...
    if (recipe_tree_list)
         connect(recipe_tree_list, SIGNAL(loosingFocus()),
                 this, SLOT( recipes_tree_loose_focus() ) );
    if (ingredientslist)
         connect(ingredientslist, SIGNAL(loosingFocus()),
                 this, SLOT( ingredients_list_loose_focus() ) );
    if (directions_text)
         connect(directions_text, SIGNAL(loosingFocus()),
                 this, SLOT( directions_list_loose_focus() ) );
    if (recipelist)
         connect(recipelist, SIGNAL(loosingFocus()),
                 this, SLOT( recipelist_list_loose_focus() ) );
    if (filter_text)
         connect(filter_text, SIGNAL(loosingFocus()),
                 this, SLOT( filter_list_loose_focus() ) );
    
    
    tab_button[0] = getUITextButtonType("general_button");
    if (tab_button[0]) {
        connect(tab_button[0], SIGNAL(pushed()), 
                this,          SLOT(switchToGeneral()));
                
        tab_button[0]->setText(tr("1 General"));
        tab_button[0]->allowFocus( false );
        
         connect(tab_button[0], SIGNAL(loosingFocus()),
                 this, SLOT( button_loose_focus() ) );
         connect(tab_button[0], SIGNAL(takingFocus()),
                 this, SLOT( button_enter_focus() ) );
    }
        
    tab_button[1] = getUITextButtonType("ingredients_button");
    if (tab_button[1]) {
        connect(tab_button[1], SIGNAL(pushed()), 
                this,          SLOT(switchToIngredients()));
                
        tab_button[1]->setText(tr("2 Ingredients"));
        tab_button[1]->allowFocus( false );
        
         connect(tab_button[1], SIGNAL(loosingFocus()),
                 this, SLOT( button_loose_focus() ) );
         connect(tab_button[1], SIGNAL(takingFocus()),
                 this, SLOT( button_enter_focus() ) );
    }   
    
    tab_button[2] = getUITextButtonType("instructions_button");
    if (tab_button[2]) {
        connect(tab_button[2], SIGNAL(pushed()), 
                this,          SLOT(switchToInstructions()));
                
        tab_button[2]->setText(tr("3 Instructions"));
        tab_button[2]->allowFocus( false );
         connect(tab_button[2], SIGNAL(loosingFocus()),
                 this, SLOT( button_loose_focus() ) );
         connect(tab_button[2], SIGNAL(takingFocus()),
                 this, SLOT( button_enter_focus() ) );
    }
        
    /*
    recipename      = new MythRemoteLineEdit(this);
        recipename->show();
	recipename->allowFocus( true );
	recipename->setGeometry(5,5,100,50);

    ingredientsname = new MythRemoteLineEdit(this);
        ingredientsname->show();
	recipename->setGeometry(5,60,100,50);

    favourite       = new MythCheckBox(this);
        favourite->show();
	recipename->setGeometry(5,120,100,50);

    */
        

}




/*
 *
 *
*/
void ModuleWindow::readFromPlayerStdout() {

    if (player_output_stream) {
        (*player_output_stream) << ( (QCString )player_process->readStdout());
        cout << "\tplayer output.... (("
             // << (( (QCString )player_process->readStdout())).data;
             << "))"
             << endl;
    }
}


/*
 *
 *
*/
void ModuleWindow::player_process_done() {


    cout << "\tModuleWindow::player_process_done() ... \n"
	 << "\tfinish data from lynx output... "
	 << endl;

}


void ModuleWindow::switchToGeneral() {
    setContext(1);
}


void ModuleWindow::switchToIngredients() {
    setContext(2);
}


void ModuleWindow::switchToInstructions() {
    setContext(3);
}

/*


void ModuleWindow::startRecording() {
    recording = TRUE;
}


void ModuleWindow::stopRecording() {
    recording = FALSE;
}

void ModuleWindow::showVolume(bool on_or_off) {
}



void ModuleWindow::newstation( int ) {
}

void ModuleWindow::previous() {
}

void ModuleWindow::next() {
}


void ModuleWindow::seeknext() {
}

void ModuleWindow::seekprev() {
}


void ModuleWindow::stop() {
}

void ModuleWindow::record() {
}

*/
