#ifndef MODULE_RECIPE_H
#define MODULE_RECIPE_H
/*
 * derived from mythgame
*/

#include <qtimer.h>
#include <qmutex.h>
#include <qvaluevector.h>
#include <qprocess.h>
#include <qfile.h>

#include <qsqldatabase.h>
#include <mythtv/mythdbcon.h>
#include <mythtv/mythwidgets.h>
#include <mythtv/dialogbox.h>
#include <mythtv/volumecontrol.h>
#include <mythtv/xmlparse.h>
#include <mythtv/uilistbtntype.h>

#include "ViewRecipeWindow.h"



class ModuleWindow : public MythThemedDialog
{
    Q_OBJECT

  public:

    //
    // Now featuring a themed constructor ... YAH!!!
    //
    typedef QValueVector<int> IntVector;
    
    ModuleWindow(MythMainWindow *parent, MSqlQuery *db, 
		    QString window_name, QString theme_filename, const char *name=0);
    ~ModuleWindow(void);

    void keyPressEvent(QKeyEvent *e);
    void updateStatus();
    int  nodeType(GenericTree *node); 

  public slots:
    // player process....
    void readFromPlayerStdout();
    void player_process_done();
    void showVolume(bool /* on_or_off */);
    void stop();
    void record();

    void goRight();
    void goLeft();
    void edit();
    void next();
    void previous();
    void seeknext();
    void seekprev();
    void enterPressed();
    void visEnable();
    void setNewContext( int key);
    void hideVolume(){showVolume(false);}
    void handleTreeListSelection(int, IntVector*);
    void handleTreeListEntry(int, IntVector*);
    void displayRecipe(QString);
    void viewRecipe();
    void displayDirections( int );
    void displayFilterText();
    void toggleFavourite();
    void fillRecipeList();
    void fillRecipeTree(QString filter=0);
    void addRecipeToList(GenericTree *);
    void numberPressAction(int );
    void cursorPgUp();
    void cursorUp();
    void cursorPgDown();
    void cursorDown();
    void updateStatus( QString );
    void closeEvent(QCloseEvent *);
    void showEvent(QShowEvent *);
    void customEvent(QCustomEvent *);
    void buildRecipeList( bool /* all */, int /* cur_list */ );
    void buildCategoryList();
    void updateRecipelistTreeTitle( GenericTree * );
    void categoryListChanged( UIListBtnTypeItem * );
    void recipeListChanged( UIListBtnTypeItem * );
    void ingredientsListChanged( UIListBtnTypeItem * );

    void switchToGeneral();
    void switchToIngredients();
    void switchToInstructions();
    
    // Focus enter events...
    void ingredients_list_enter_focus();
    void recipes_tree_enter_focus();
    void directions_list_enter_focus();
    void recipelist_list_enter_focus();
    void filter_list_enter_focus();
    void button_enter_focus();

    // Focus loose events...
    void ingredients_list_loose_focus();
    void recipes_tree_loose_focus();
    void directions_list_loose_focus();
    void recipelist_list_loose_focus();
    void filter_list_loose_focus();
    void button_loose_focus();

    
  signals:
  
    void dummy();   // debugging

  private:

    // player process variables... ----
    QString      playfile;
    QProcess    *player_process;
    QString      player_output;
    QFile        *player_output_file;
    QTextStream  *player_output_stream;

    //--------------------------------

    QByteArray    player_output_buffer;
    bool          mute;
    // --------------------------------

    QString statusString;
    int directions_cnt, directions_top;
    QStringList cur_directions;

    MSqlQuery *db;

    void wireUpTheme();

    int currentTime, maxTime;
    int m_InColumn;

    // Metadata *curMeta;

    bool lcd_volume_visible;
    bool muted;
    
    int  widget_focus;

    VolumeControl   *volume_control;

	
    //
    //  Theme-related "widgets"
    //


    UIManagedTreeListType *recipe_tree_list;
    GenericTree           *recipe_tree_root;
    GenericTree           *recipe_tree_data;

    XMLParse *recipeTheme;
    XMLParse *theme;

    QRect     categorylistRect;
    QRect     recipelistRect;
    QRect     ingredientslistRect;
    QRect     recipetreelistRect;
    
    UIBlackHoleType       *ingredient_blackhole;

    UIListBtnType         *categorylist;
    UIListBtnType         *recipelist;
    UIListBtnType         *ingredientslist;
    
    UIImageType           *recipe_image;
    UIImageType           *info0_image;
    UIImageType           *info1_image;
    UIImageType           *info2_image;
    UIImageType           *info3_image;
    UIImageType           *info4_image;
    
    UITextType            *author_text;
    UITextType            *source_text;
    UITextType            *serving_text;
    UITextType            *comment_text;
    UITextType            *extra_comment_text;
    UITextType            *directions_text;
    UITextType            *ingredient_text;
    UITextType            *recipetreelisttitle_text;
    UITextType            *filter_text;
    UITextType            *total_recipe_text;
    UITextType            *current_recipe_text;
    
    UITextButtonType      *tab_button[3];
    
    UIStatusBarType       *volume_status;
    UIPushButtonType      *mute_button;

    MythCheckBox          *favourite;
    MythRemoteLineEdit    *recipename;
    MythRemoteLineEdit    *ingredientsname;

    bool                  recipeshowfavourites;
    QString               recipenamefilter;
    QString               recipecoursefilter;
    QString               recipecuisinefilter;
    QString               recipeingredientfilter;

};


#endif
