#include <mythtv/mythcontext.h>

#include "Modulesettings.h"
#include <qfile.h>
#include <qdialog.h>
#include <qcursor.h>
#include <qdir.h>
#include <qimage.h>

// General Settings



static HostCheckBox *RecipeShowFavourites() {
    HostCheckBox *gc = new HostCheckBox("RecipeShowFavourites");
    
        gc->setLabel(QObject::tr("Show Only Favourites"));
        gc->setValue(false);
        gc->setHelpText(QObject::tr("Limit Recipes listed to only those tagged "
                    "as \"favourite\""));
    return gc;
};



static  HostSlider *RecipeGetNumRecipesAtATime() {
    HostSlider *gc = new HostSlider("RecipeGetNumRecipesAtATime", 20, 1000, 20);

        gc->setLabel(QObject::tr("Number Of Recipes To Get At A Time "));
        gc->setValue(100);
        gc->setHelpText(QObject::tr("Set the number of recipes to get at a time "
                                "from scanning web pages for recipes..."));
    return gc;
};



static  HostCheckBox *RecipePurgeWhenScraping() {
    HostCheckBox *gc = new HostCheckBox("RecipePurgeWhenScraping");

        gc->setLabel(QObject::tr("Purge When Scraping"));
        gc->setValue(false);
        gc->setHelpText(QObject::tr("Purge recipe tables before scraping starts\n"
				"only do this if you want to do a new "
				"scrape... It will remove all your existing "
				"recipes from the tables..."));
    return gc;
};



static  HostLineEdit *RecipeCourseStart() {
    HostLineEdit *gc = new HostLineEdit("RecipeCourseStart");

        gc->setLabel(QObject::tr("Starting Recipe Course "));
        gc->setValue("1");
        gc->setHelpText(QObject::tr("Starting recipe course for scraping recipes "
				"from www.cooking.com\nmake sure you uncheck "
				"purge option so your recipes are added to "
				"the existing recipes in the database."));
    return gc;
};



static  HostLineEdit *RecipePageStart() {
    HostLineEdit *gc = new HostLineEdit("RecipePageStart");

        gc->setLabel(QObject::tr("Starting Course Page"));
        gc->setValue("1");
        gc->setHelpText(QObject::tr("Starting course at this page when scraping "
				"recipes from www.cooking.com\nmake sure you "
				"uncheck purge option so your recipes are "
				"added to the existing recipes in the "
				"database."));
    return gc;
};



static  HostLineEdit *RecipeCourseEnd() {
    HostLineEdit *gc = new HostLineEdit("RecipeCourseEnd");

        gc->setLabel(QObject::tr("Ending Recipe Course "));
        gc->setValue("14");
        gc->setHelpText(QObject::tr("Ending recipe course for scraping recipes "
				"from www.cooking.com\nmake sure you uncheck "
				"purge option so your recipes are added to "
				"the existing recipes in the database."));
    return gc;
};


static  HostLineEdit *RecipeLoadLocation() {
    HostLineEdit *gc = new HostLineEdit("RecipeLoadLocation");

        gc->setLabel(QObject::tr("Path to Recipe Files"));
        gc->setValue(PREFIX "/share/mythtv/mythRecipe/xml");
        gc->setHelpText(QObject::tr("Directory where Recipes in RecipeML and "
				"mythRecipeURL forms/files are kept."));
    return gc;
};


static  HostCheckBox *RecipeImageOnly() {
    HostCheckBox *gc = new HostCheckBox("RecipeImageOnly");

        gc->setLabel(QObject::tr("Display Recipes with Images Only"));
        gc->setValue( false );
        gc->setHelpText(QObject::tr("Display only those recipes that has an image "
				"associated with them."));
    return gc;
};


static  HostComboBox *RecipeSetupRecipeTables() {
    HostComboBox *gc = new HostComboBox("RecipeSetupRecipeTables");

        gc->setLabel(QObject::tr("Setup Recipe Table Now ?"));
        gc->addSelection( "Purge Once" );
        gc->addSelection( "No Action" );
        gc->addSelection( "Purge EveryTime" );
        gc->setHelpText(QObject::tr("Purge and setup the Recipe tables to default\n"
                            "No Action: take no action at start of module\n"
                            "Purge Once: purge at first start of module\n"
                            "Purge EveryTime: purge each time module starts"));
    return gc;
};


static  HostCheckBox *RecipeMusicDisableIfFM() {
    HostCheckBox *gc = new HostCheckBox("RecipeMusicDisableIfFM");

        gc->setLabel(QObject::tr("Disable If FM available ?"));
        gc->setValue( false );
        gc->setHelpText(QObject::tr("Disable music if FM available.\n.Start FM "
				"music if FM is available and disable Music "
				"from playing"));
    return gc;
};


static  HostCheckBox *RecipeMusicSelectRandomURL() {
    HostCheckBox *gc = new HostCheckBox("RecipeMusicSelectRandomURL");

        gc->setLabel(QObject::tr("Select Random URL to play ?"));
        gc->setValue( false );
        gc->setHelpText(QObject::tr("Should a random URL be played each time "
				"music play is requested ?" ));
    return gc;
};


static  HostLineEdit *RecipeMusicProgram() {
    HostLineEdit *gc = new HostLineEdit("RecipeMusicProgram");

        gc->setLabel(QObject::tr("Path to RecipeMusicProgram  Files"));
        gc->setValue("mpg123");
        gc->setHelpText(QObject::tr("Compatible program to play your favourite "
				"urls for internet radio while you cookup your"
				" recipes...."));
    return gc;
};


static  HostComboBox *RecipeMusicURLList() {
    HostComboBox *gc = new HostComboBox("RecipeMusicURLList");

        gc->setLabel(QObject::tr("Add to URL For MusicProgram "));
        gc->addSelection( "http://64.62.252.134:6310/" );
        gc->addSelection( "http://69.93.28.52:9009/" );
        gc->addSelection( "http://69.93.28.52:9009/" );
        gc->setHelpText(QObject::tr("Add Your urls for the RecipeMusicprogram "
				"here\nselecton shown will be the default "
				"default URL to play..."));
    return gc;
};


static  HostCheckBox *RecipeMusicLeaveOpenWhenExit() {
    HostCheckBox *gc = new HostCheckBox("RecipeMusicLeaveOpenWhenExit");

        gc->setLabel(QObject::tr("Leave Player going when Exit ?"));
        gc->setValue( false );
        gc->setHelpText(QObject::tr("Should the player stays playing when recipe "
				"module exits or should the player remain on"));
    return gc;
};


static  HostCheckBox *RecipeMusicEnableMusic() {
    HostCheckBox *gc = new HostCheckBox("RecipeMusicEnableMusic");

        gc->setLabel(QObject::tr("Enable Music Player ?"));
        gc->setValue( false );
        gc->setHelpText(QObject::tr("Should the music player plays your URLS when "
				"mythRecipe starts" ));
    return gc;
};



MythModuleSettings::MythModuleSettings() {
    VerticalConfigurationGroup* page1 = new VerticalConfigurationGroup(false);
    page1->setLabel(QObject::tr("MythRecipe Settings -- General"));
    page1->addChild(RecipeSetupRecipeTables());
    page1->addChild(RecipeShowFavourites());
    page1->addChild(RecipePurgeWhenScraping());
    page1->addChild(RecipeGetNumRecipesAtATime());
    page1->addChild(RecipeCourseStart());
    page1->addChild(RecipePageStart());
    page1->addChild(RecipeCourseEnd());
    page1->addChild(RecipeLoadLocation());
    page1->addChild(RecipeImageOnly());
    addChild(page1);

    VerticalConfigurationGroup* page2 = new VerticalConfigurationGroup(false);
    page2->setLabel(QObject::tr("MythRecipe Settings -- Music"));
    page2->addChild(RecipeMusicEnableMusic());
    page2->addChild(RecipeMusicProgram());
    page2->addChild(RecipeMusicURLList());
    page2->addChild(RecipeMusicDisableIfFM());
    page2->addChild(RecipeMusicSelectRandomURL());
    page2->addChild(RecipeMusicLeaveOpenWhenExit());
    addChild(page2);
}
