#include <qapplication.h>
#include <qregexp.h>
#include <stdlib.h>
#include <iostream>
	using namespace std;
#include <qdom.h>
#include <qdir.h> 

#include "ParseML.h"

#include "menu.h"



int ParseML::parseDirections(QDomNode node, QStringList &dir) {
    QDomNode n = node.firstChild();

    while( !n.isNull() ) {
        QDomElement e = n.toElement();    // try to convert the 
        
        if( !e.isNull() ) {
            if (e.tagName().lower()=="step") {
                QDomText e = n.firstChild().toText(); 
                // cout << "\t\tstep - " << e.nodeValue() << endl;
                dir.append( e.nodeValue() );
            } else if (e.tagName().lower()=="notes") {
            } else
                cout << "Invalid node... ("<<e.tagName()<<"\n";
        }
        n = n.nextSibling();
    }
        
    return 0;
}

int ParseML::parseHeadCategory(QDomNode node, QStringList &cat) {
    QDomNode n = node.firstChild();

    while( !n.isNull() ) {
        QDomElement e = n.toElement();    // try to convert the 
        
        if( !e.isNull() ) {
            if (e.tagName().lower()=="cat") {
                QDomText e = n.firstChild().toText(); 
                cat.append( e.nodeValue() );
            } else if (e.tagName().lower()=="notes") {
            } else
                cout << "Invalid node... ("<<e.tagName()<<"\n";
        }
        n = n.nextSibling();
    }
        
    return 0;
}



int ParseML::parseHeadTitle(QDomNode node, QString &title) {
    QDomNode n = node.firstChild();
    while( !n.isNull() ) {
        QDomText e = n.toText();                 // try to convert the 
                                                // node to an element.
        if( !e.isNull() ) {
            title = e.nodeValue();              // the node really is 
                                                    // an element.
        }
        n = n.nextSibling();
    }
    return 0;        
}


int ParseML::parseHead(QDomNode node, Header &head) {
	    QDomNode n = node.firstChild();

	    head.clearCategory();

	    while( !n.isNull() ) {
		QDomElement e = n.toElement();                 // try to convert the 
							       // node to an element.
		if( !e.isNull() ) {
		    if (e.tagName().lower()=="title") {
			QDomText e = n.firstChild().toText(); 
			head.setTitle( e.nodeValue() );
		    } else if (e.tagName().lower()=="categories") {
			QDomText e = n.firstChild().toText(); 
			QString category = e.nodeValue();
			if (category.isEmpty() || category=="") {
			    QStringList catlist;
			    parseHeadCategory( n, catlist);
			    for (QStringList::Iterator it=catlist.begin();
						       it!=catlist.end(); ++it){
				head.addCategory( (*it) );
			    }
			} else
			    head.addCategory( category );
		    } else if (e.tagName().lower()=="yield") {
                        QString str;
                        Amount amt;

			QDomText e = n.firstChild().toText(); 
                          
			str = e.nodeValue();

                        if ( (str.isEmpty() || str.isNull()) ) {
                           // new amount node
                        } else {
                            amt.setQuantity( str.toFloat() );
                        }
			head.setYield( amt );
		    } else
			cout << "Invalid node... ("<<e.tagName()<<"\n";
		}
		n = n.nextSibling();
	    }
    return 0;
	}



int ParseML::parseIngItem(QDomNode node, IngItem &ing_item) {
    QDomNode n = node.firstChild();

    while( !n.isNull() ) {
        QDomElement e = n.toElement();   // try to convert the 
					       // node to an element.
	if( !e.isNull() ) {
	    if (e.tagName().lower()=="amt") {
                Amount amt;
                parseAmount(n, amt);
                ing_item.addAmount(amt);
	    } else if (e.tagName().lower()=="modifier") {
		QDomText e = n.firstChild().toText(); 
		//
                cout << "parsing the(IngItem)  modifier..." << endl;
	    } else if (e.tagName().lower()=="item") {
		QDomText e = n.firstChild().toText(); 
		//
                ing_item.setItem( e.nodeValue() );
                // cout << "parsing the (IngItem) item..." << endl;
	    } else if (e.tagName().lower()=="prep") {
		QDomText e = n.firstChild().toText(); 
		//
                cout << "parsing the (IngItem) prep..." << endl;
	    } else if (e.tagName().lower()=="ing-note") {
		QDomText e = n.firstChild().toText(); 
		//
                cout << "parsing the (IngItem) ing-note..." << endl;
	    } else if (e.tagName().lower()=="prodcode") {
		QDomText e = n.firstChild().toText(); 
		//
                cout << "parsing the (IngItem) prodcode..." << endl;
	    } else if (e.tagName().lower()=="alt-ing") {
		QDomText e = n.firstChild().toText(); 
		//
	    } else
		cout << "Invalid Ing Item.... ("<<e.tagName()<<"\n";
	} else 
            cout << " Ing Item Node is NULL.... \n";

	n = n.nextSibling();
    }
    return 0;
}


int ParseML::parseAmount(QDomNode node, Amount &amt) {
    float qty;
    QString unit, size;
    bool foundqty=false;

    QDomNode n = node.firstChild();
    QStringList sl;
        
    while( !n.isNull() ) {
        QDomElement e = n.toElement();                 // try to convert the 
        // node to an element.
        if( !e.isNull() ) {
            if (e.tagName().lower()=="qty") {
                QDomText e = n.firstChild().toText(); 
                // cout << "\t\tqty - " << e.nodeValue() << endl;
                foundqty=true;
                qty = e.nodeValue().toFloat();
                amt.setQuantity( e.nodeValue().toFloat() );
            } else if (e.tagName().lower()=="unit") {
                QDomText e = n.firstChild().toText(); 
                // cout << "\t\tunit - " << e.nodeValue() << endl;
                unit = e.nodeValue();
                amt.setUnit( e.nodeValue() );
            } else if (e.tagName().lower()=="size") {
                QDomText e = n.firstChild().toText(); 
                // cout << "\t\tsize - " << e.nodeValue() << endl;
                size = e.nodeValue();
                amt.setSize( e.nodeValue() );
            } else if (e.tagName().lower()=="range") {
                // parseRange( n.firstChild(), r1, r2 ); 
                // cout << "\t\trange - " << e.nodeValue() << endl;
                // size = e.nodeValue();
                // amt.setRange( r1, r2 );
            } else
                cout << "Invalid node...\n";
        }
        n = n.nextSibling();
    }
    return 0;
}


int ParseML::parseIngredients(QDomNode node, 
                              QValueList<Ingredient> &ingredients) {
    QDomNode n = node.firstChild();
    QString amount;
    QStringList sl;

    while( !n.isNull() ) {
        QDomElement e = n.toElement();   // try to convert the 
                                         // node to an element.
        if( !e.isNull() ) {
            if (e.tagName().lower()=="ing-div") {
                Ingredient ing;
                // cout << "<ing-div>" << endl;
                QDomNode children = n.firstChild();
                while (!children.isNull()) {
                    QDomElement de = children.toElement();
                    if (de.tagName().lower()=="ing") {
                        IngItem item;
                        // cout << "calling <ing-div> parseIngItem" << endl;
                        parseIngItem(children, item);
                        ing.addIngItem( item );
                    } else if (de.tagName().lower()=="title") {
                        //
                        QDomText e = children.firstChild().toText(); 
                          
                        // cout <<"found <ing-div> Title"<<e.nodeValue()<<endl;
                        ing.setTitle( e.nodeValue() );
                    } else if (de.tagName().lower()=="description") {
                        // cout << "found <ing-div> description" << endl;
                        //
                        QDomText e = children.firstChild().toText(); 
                          
                        ing.setDescription( e.nodeValue() );
                    } else if (de.tagName().lower()=="note") {
                        IngItem item;
                        // cout << "found <ing-div> note" << endl;
                        //
                        QDomText e = children.firstChild().toText(); 
                          
                        item.addNote( e.nodeValue() );
                        ing.addIngItem( item );
                    } else {
                        cout << "\tinvalid <ing-div> ingredient type tag "
                        << de.tagName() << endl;
                        break;
                    }
                    children = children.nextSibling();
                }
                ingredients.append( ing );
            } else if (e.tagName().lower()=="ing") {
                Ingredient ing;
                IngItem item;
                // cout << "calling parseIngItem (ing case)" << endl;
                parseIngItem(n, item);
                ing.addIngItem( item );
                ingredients.append( ing );
            } else if (e.tagName().lower()=="note") {
                Ingredient ing;
                IngItem item;
                // cout << "calling parseIngItem (note case)" << endl;
                QDomText e = n.firstChild().toText(); 
                          
                item.addNote( e.nodeValue() );
                ing.addIngItem( item );
                ingredients.append( ing );
            } else {
                cout << "\tInvalid Ingredient tag name\n";
                break;
            }
        }
        n = n.nextSibling();
    }
    return 0;
}



int ParseML::parseMenu(QDomNode node, Menu &menu) {
    QDomNode n = node.firstChild();
    QStringList description;
    QStringList head;

    bool foundhead=false;

    while( !n.isNull() ) {
        QDomElement e = n.toElement(); 
        if( !e.isNull() ) {
            // cout << e.tagName() << endl; 
            if (e.tagName().lower()=="head") {
                Header head;
                parseHead(n, head);
                foundhead=true;
                menu.setHeader( head );
            } else if (e.tagName().lower()=="recipe") {
                Recipe recipe;
                parseRecipe(n, recipe);
                menu.addRecipe( recipe );
            } else
                cout << "Invalid node...\n";
        }
    }
    return 0;
}



int ParseML::parseRecipe(QDomNode node, Recipe &recipe) {
    QDomNode n = node.firstChild();
    bool foundhead=false;

    while( !n.isNull() ) {
        QDomElement e = n.toElement(); 
        if( !e.isNull() ) {
            // cout << e.tagName() << endl; // the node really is an element.
            if (e.tagName().lower()=="head") {
                Header head;
                parseHead(n, head);
                foundhead=true;
		recipe.setHeader( head);
            } else if (e.tagName().lower()=="ingredients") {
		QValueList<Ingredient>  ingredients;
                // cout << "Ingredients - " << e.tagName() << endl;
                parseIngredients(n, ingredients);
                for (QValueList<Ingredient>::Iterator it=ingredients.begin();
                                                 it!=ingredients.end(); ++it){
                    recipe.addIngredient( (*it) );
                }
            } else if (e.tagName().lower()=="directions") {
                QStringList directions;
                // cout << "Directions - " << e.tagName() << endl;
                parseDirections(n, directions);
                for (QStringList::Iterator it=directions.begin();
                                           it!=directions.end(); ++it){
                    recipe.addDirection( (*it) );
                }
            } else
                cout << "Invalid node...\n";
        }
        n = n.nextSibling();
    }
    return 0;
}


ParseML::ParseML() {
	
}


int ParseML::ParseMLFile( QString filename, Menu &menu ) {

    QDomDocument doc( "myrecipe" );
    QFile file( filename );

    if ( !file.open( IO_ReadOnly ) ) {
        cout << "unable to open file ..." << filename << "\n";
        return 1;
    }

    if ( !doc.setContent( &file ) ) {
        file.close();
        cout << "unable to open/read xml document...i" << filename << "\n";
        return 2;
    }
    file.close();

     
    // print out the element names of all elements that are direct children
    // of the outermost element.
    QDomElement docElem = doc.documentElement();

    QDomNode n = docElem.firstChild(); // .firstChild();
     
    while( !n.isNull() ) {
        QDomElement e = n.toElement(); //try to convert the node to an element.
        if( !e.isNull() ) {
            // cout << e.tagName() << endl; // the node really is an element.
            if (e.tagName().lower()=="head") {
                Header head;
                // cout << "\tparsing a MENU head element...." << endl;
                parseHead(n, head);
                menu.setHeader( head );
            } else if (e.tagName().lower()=="recipe") {
                Recipe recipe;
                // cout << "\tparsing a Recipe element...." << endl;
                parseRecipe(n, recipe);
                menu.addRecipe( recipe );		
            } else
                cout << "Invalid node... " << e.tagName() << "\n";
        }
        n = n.nextSibling();
    }
     
    return 0;
}

