#include <mythtv/mythcontext.h>
#include <mythtv/mythwidgets.h>
#include <mythtv/lcddevice.h>
#include <mythtv/xmlparse.h>


#include "RecipeDB.h"
#include "GetLynxFile.h"
#include "menu.h"

#define RECIPEEXISTS   00001


/*
 *
 *
*/
RecipeDB::RecipeDB(QObject *parent, const char *name, MSqlQuery *ldb)
           : QObject(parent, name) {
    if (ldb) {
        db = ldb;
    } else {
        cout << "RecipeDB::RecipeDB()... no databse found using default...";
        // db = QSqlDatabase::database();
    }
}



/*
 *
 * add a Recipe class filled with information on the recipe to the database...
*/
int RecipeDB::addRecipeToDatabase( Recipe &recipe ) {
    Header head = recipe.getHeader();
    QString recipename = head.getTitle();
    QString yield = head.getYield().value();
    QValueList<QString> &categories = head.getCategory();
    QValueList<Ingredient> &ingredients = recipe.getIngredient();
    QValueList<QString> &directions = recipe.getDirection();
     
    int recipeid = 0;
    int courseid = 0;
    int cnt=0;


    if ( (recipeid=addRecipe( head )) <0 ) {
        switch (recipeid) {
            case -2 :
                     cout << "(-1) unable to add recipe" << endl;
                     break;
            case -1 :
                     cout << "(-2) unable to add recipe" << endl;
                     break;
        }
        return recipeid;
    }

    // cout << "processing Categories in recipe " << endl;
    for (QValueList<QString>::Iterator it=categories.begin();
                                      it!=categories.end();++it ) {
        if ( (courseid=addCourse((*it), 1))>=0 ) {
            if ( (courseid=addRecipeToCourse(recipeid, courseid))>=0 ) {
            } else {
               cout << "unable to add recipe to course " << (*it) << endl;
            }
        } else {
           cout << "unable to add course " << (*it) << endl;
        }
    }

    // int cnt=1;
    // cout << "processing Ingredients in recipe " << endl;
    for (QValueList<Ingredient>::Iterator it=ingredients.begin();
                                          it!=ingredients.end();++it ) {
        addIngredients(recipeid, (*it));
    }

    // cout << "processing Directions in recipe " << endl;
    cnt=0;
    for (QValueList<QString>::Iterator it=directions.begin();
                                      it!=directions.end();++it ) {
        QStringList sl = QStringList::split(". ", (*it));
        for (QStringList::Iterator line=sl.begin();
                                  line!=sl.end(); ++line ) {
            addInstructionLine(recipeid, cnt++, (*line));
        }
    }
    
    return recipeid;
}


/*
 * Add an instruction line to the database.
 *
*/
int RecipeDB::addInstructionLine( int recipeid, int linenumber, QString &step ) {
    QString insert;
    QSqlQuery query;

    //
    // #instructions are here....
    insert = QString("INSERT INTO instructions values(%1,%2,\"%3\")")
                                                       .arg(recipeid)
                                                       .arg(linenumber)
                                                       .arg(step);
    // query = db->exec(insert);
    db->prepare(insert);
    db->exec();

    return 0;
}


/*
 * Get the name of the author with authorid id
 *
*/
QString RecipeDB::getAuthorName( int authorid ) {
    QString select;
    QSqlQuery query;

    select.sprintf("SELECT user_name FROM users  WHERE "
              "user_id = %d ", authorid );

    // query = db->exec(select);
   db->prepare(select);
    db->exec();
            
    if (db->next()) {
        QString str = db->value(0).toString(); 
        if (str.isEmpty() || str.isNull() ) {
            return "unKnown";
        } else 
            return str;
    } else 
        return "unKnown";
}


/*
 * return the author id for the named author..
 * If author does not exist in the database set addifnot exist to tru and 
 * he will be added
*/
int RecipeDB::getAuthorId( QString &author, bool addifnotexist ) {
    QString select;
    QString insert;
    QSqlQuery query;

    if ( author.isEmpty() ||author.isNull())
        return 0;

    select = ("SELECT user_id FROM users  WHERE "
              "user_name = \"" + author + "\" " );

    // query = db->exec(select);
    db->prepare(select);
    db->exec();
            
    if (db->next()) {
        return  query.value(0).toInt(); 
    } else {
        if (addifnotexist) {

            insert = "INSERT INTO users ( user_name) "
                     "values(0, \"" + author + "\") ;";
    
            // query = db->exec(insert);
            db->prepare(insert);
            db->exec();

            select = ("SELECT user_id FROM users  WHERE "
                      "user_name = \"" + author + "\" " );
    
            // query = db->exec(select);
            db->prepare(select);
            db->exec();
    	 
            if (!db->next()) {
                cout << "\terror trying to add author to database"
                     << endl;
	        return 0;
            } else
                return db->value(0).toInt(); 
        } else
            return 0; 
    }
    return 0;
}


/*
 * Add ingredients in the class to the database
 *
*/
int RecipeDB::addIngredients(int recipeid, Ingredient &ingredients ) {
    int cnt=0;
    QSqlQuery query;
    QString insert;

    QString &title = ingredients.getTitle();
    QString &descr = ingredients.getDescription();
    QValueList<IngItem> &ing_items = ingredients.getIngItems();

    if (!(title.isEmpty()||title.isNull()|| title=="none") ){
        cout << "Ingredient Title: ";
        cout << title;
        cout << endl;
        insert = QString("INSERT INTO ingredients " 
               "(recipe_id,display_order,ingredient_type,name, qty, unit_id) "
        "values(%1,       %2,           %3,          \"%4\", \"%5\", \"%6\") ")
            .arg(recipeid)
            .arg(cnt++)
            .arg(2)
            .arg(title)
            .arg(0)
            .arg(0);

        // query = db->exec(insert);
        db->prepare(insert);
        db->exec();
    }
    if (!(descr.isEmpty()||descr.isNull()||descr=="none") ) {
        cout << "Description : ";
        cout << descr;
        cout << endl;
    }

    for ( QValueList<IngItem>::Iterator it=ing_items.begin();
		              it!=ing_items.end(); ++it ) {
        QValueList<Amount> &amt = (*it).getAmount();

        insert = QString("INSERT INTO ingredients " 
               "(recipe_id,display_order,ingredient_type,name, qty, unit_id) "
        "values(%1,       %2,           %3,          \"%4\", \"%5\", \"%6\") ")
            .arg(recipeid)
            .arg(cnt++)
            .arg(1)
            .arg((*it).getItem())
            .arg(amt[0].qty())
            .arg(getUnitId(amt[0].unit()));

        // query = db->exec(insert);
        db->prepare(insert);
        db->exec();
    }
    return 0;
}


/*
 * Add the main information about the recipe tro the database
 * all the main information is held in the Header class...
 *
*/
int RecipeDB::addRecipe( Header &recipehead ) {
    QString insert;
    int authorid=0;
    int recipeid=0;

    authorid = getAuthorId( recipehead.getAuthor() );


    recipeid = getRecipeId(recipehead.getTitle());

    if (recipeid<(-1)) {
        cout << "RecipeDB::addRecipe()... invalid recipe name" << endl;
    } else if (recipeid<0) { 
	insert = QString("INSERT INTO recipes (name, yield_qty, yield_units, "
                         "source, author_user_id,create_utime,mod_utime) "
          " values(\"%1\", \"%2\", \"%3\", \"%4\", \"%5\", \"%6\", \"%7\" ) ")
                 .arg(recipehead.getTitle())
                 .arg(recipehead.getYield().qty())
                 .arg(recipehead.getYield().unit())
                 .arg(recipehead.getSource())
                 .arg(authorid)
                 .arg(QDateTime::currentDateTime().toString())
                 .arg(QDateTime::currentDateTime().toString());
         
        // QSqlQuery query = db->exec(insert);
        db->prepare(insert);
        db->exec();
	
        if ( (recipeid = getRecipeId(recipehead.getTitle()))<0) {
            cout << "RecipeDB::addRecipe()... error trying to add recipe to database"
		 << endl;
        }
    } else
        cout << "RecipeDB::addRecipe()... found recipe not adding" << endl;

    return recipeid;
}


/*
 * Associates a recipe to the course 
 * return the cours id or the new course id added
 *
*/
int RecipeDB::addRecipeToCourse( int recipeid, int course ) {
    QString select;
    QString insert;

    select = QString("SELECT category_id FROM category_entries  WHERE "
             "category_id = %1 AND recipe_id = %2 ")
              .arg(course)
              .arg(recipeid);

    // QSqlQuery query = db->exec(select);
    db->prepare(select);
    db->exec();
            
    if ( !(db->next()) )  {
        insert = QString("INSERT INTO category_entries (category_id, recipe_id) "
                 "values(%1, %2) ;")
                  .arg(course)
                  .arg(recipeid);

        // query = db->exec(insert);
        db->prepare(insert);
        db->exec();

        return course;
    } else
        return db->value(0).toInt(); 
}



/*
 * Add course to the database 
 * if parent = 0 course is a course
 * if parent !=0 course is representing a cuisine for the parent course
 * returns the course id or the course id of the newly addec course/cuisine
 *
*/
int RecipeDB::addCourse( QString &course, int parent ) {
    QString select;
    QString insert;
    int courseid;

    if ( course.isEmpty() || course.isNull())
        return -2;
	    
    select = QString("SELECT category_id FROM categories  WHERE "
             "name = \"%1\" AND parent = \"%2\" ")
              .arg(course)
              .arg(parent);

    // QSqlQuery query = db->exec(select);
    db->prepare(select);
    db->exec();
            
    if ( (db->next()) )  {
        courseid =  db->value(0).toInt(); 
    }  else {
        insert = QString("INSERT INTO categories (parent, name) "
                 "values(%1, \"%2\") ;")
                  .arg(parent)
                  .arg(course);

        // query = db->exec(insert);
        db->prepare(insert);
        db->exec();

        select = QString("SELECT category_id FROM categories  WHERE "
             "name = \"%1\" AND parent = %2 ")
              .arg(course)
              .arg(parent);

        // query = db->exec(select);
        db->prepare(select);
        db->exec();
	 
        if (!db->next()) {
            cout << "\terror trying to add course to database"
		 << endl;
	    return -1;
        }

        courseid = db->value(0).toInt(); 

    }
    return courseid;
}


/*
 * Get the id for the named unit of measure
 * if addifnotexist is true then the unit is added to the database for 
 * future use
 * return the unit id fornd or the newly inserted unit id
 *
*/
int RecipeDB::getUnitId( QString &unit, bool addifnotexist ) {
    QString select;
    QString insert;

    if ( unit.isEmpty() || unit.isNull()) {
        return 0;
    }

    select = ("SELECT unit_id FROM units  WHERE "
		    "name LIKE  \'\%" + unit + "\%\' " ) ;

    // QSqlQuery query = db->exec(select);
    db->prepare(select);
    db->exec();
            
    if (db->next()) {
        return  db->value(0).toInt(); 
    } else {
        if (addifnotexist) {
            insert = ("INSERT INTO units (name, abbreviation) "
			"values(\"" + unit + "\", \"" + unit + "\") ;" );
            // query = db->exec(insert);
            db->prepare(insert);
            db->exec();

            select = ("SELECT unit_id FROM units  WHERE "
                      "name LIKE  \'\%" + unit + "\%\' " ) ;
            cout << "RecipeDB::getUnitId()... inserting ("<< unit << ") " ;
            
            // QSqlQuery query = db->exec(select);
            db->prepare(select);
            db->exec();
            
            if (db->next()) {
                return  db->value(0).toInt(); 
            } else  {
                return 0;
            }
        } else
           return 0; 
    }

}


/*
 * Search an return the id for the course given
 * if parent !=0 then course is a cuisine
 * id is returned...
 *
*/
int RecipeDB::getCourseId( QString coursename, int parent ) {
    QString select;


    if ( coursename.isEmpty() ||coursename.isNull())
        return -2;

    select = QString("SELECT category_id FROM categories  WHERE "
		    "name = \"%s\" AND parent = %d ")
                     .arg(coursename)
                     .arg(parent);

    // QSqlQuery query = db->exec(select);
    db->prepare(select);
    db->exec();
            
    if (db->next()) {
        return  db->value(0).toInt(); 
    } else {
        return -1; 
    }
}


/*
 * Search and returned the id of the recipe named
 * return recipe's id
 *
*/
int RecipeDB::getRecipeId( QString &recipename ) {
    QString select;


    if ( recipename.isEmpty() || recipename.isNull() )
        return -2;
	    
    select = ("SELECT recipe_id FROM recipes WHERE "
		    "name = \"" + recipename + "\" ") ;

    // QSqlQuery query = db->exec(select);
    db->prepare(select);
    db->exec();

    if ( db->next() ) {
        return db->value(0).toInt(); 
    } else {
        return -1;
    }
}


/*
 * Add the recipes found in the menu class to the database
 * add the menu information to the menu tables as well
 * retunr the menuid of the menu or the newly inserted menu
 *
 *
*/
int RecipeDB::addRecipeToDatabase( Menu &menu ) {
    int recipeid=0;
    QValueList<Recipe> recipes = menu.getRecipe();
    MythProgressDialog *pd1 = new MythProgressDialog("Adding recipes to database",
		                                      recipes.size() );

    int cnt=1;
    // cout << "RecipeDB::addRecipeToDatabase(Menu)... "
    //      << "processing recipes in menu " << endl;
    for (QValueList<Recipe>::Iterator it=recipes.begin();
                                      it!=recipes.end();++it ) {
        Header head = (*it).getHeader();

        recipeid = addRecipeToDatabase( (*it) );

        if (pd1)
            pd1->setProgress( ++cnt );
    }

    if (pd1)
        delete pd1;

    return 0;
}


/*
 *
 *
*/
RecipeDB::~RecipeDB(void) {
	
}


