#include <mythtv/mythcontext.h>
#include <mythtv/mythwidgets.h>
#include <mythtv/lcddevice.h>
#include <mythtv/xmlparse.h>
#include <mythtv/mythdbcon.h>

#include <qurl.h>
#include <qsqlquery.h>
#include <mythtv/mythdbcon.h>

#include "ScanSiteWindow.h"
#include "DisplayStatus.h"

#include "GetLynxFile.h"

#define NOLINKFOUND         "@"
#define RECIPESEPARATOR     "~~"
#define MAXCOURSES          17
#define MAXRECIPESATATIME   1000
#define MINRECIPESATATIME   20 
#define RECIPESPERPAGE      20



/*
 *
 *
*/
ScanSiteWindow::ScanSiteWindow(MythMainWindow *p, MSqlQuery *ldb, 
               QString window_name, QString theme_filename, const char *name)
           : MythThemedDialog(p, window_name, theme_filename, name){
    
    QDir tmpdir;
    QString s_dir = PREFIX "/share/mythtv/themes/default/recipes/.";

    parent=p;

    //  A few internal variable defaults

    if (ldb) {
        db = ldb;
    } else
        db = NULL;
    
    if (!tmpdir.cd( s_dir )) {
        cout << tr("unable to change directory to (%1)") 
                   .arg(s_dir.data())
             << endl;
    }

    wireUpTheme();

    if (!db) {
	 MythPopupBox::showOkPopup(parent,
              QObject::tr("Database not available."),
              QObject::tr("Cannot scan for recipes... "));
        return;
    }

    // setup temporary storage structure/tree
    //
    if (tmpdir.mkdir("/tmp/mythRecipe")) {
        if (!tmpdir.mkdir("/tmp/mythRecipe/recipepages")) {
	    cout << tr("unable to create %1")
	              .arg("/tmp/mythRecipe/recipepages...")
	         <<endl;
        }
        if (!tmpdir.mkdir("/tmp/mythRecipe/coursepages")) {
            cout << tr("unable to create %1")
	                  .arg("/tmp/mythRecipe/recipes/coursepages...")
	             << endl;
        }
        if (!tmpdir.mkdir("/tmp/mythRecipe/images")) {
            cout << tr("unable to create %1")
	                  .arg("/tmp/mythRecipe/recipes/images...")
	             << endl;
        }
    }  else 
       cout << "\t\tunable to create /tmp/mythRecipe..." << endl;
    

    pd0 = NULL;
    pd1 = NULL;
    pd2 = NULL;

    recipes_at_atime = MINRECIPESATATIME;

    scanSites();
}



/*
 *
 *
*/
void ScanSiteWindow::scanSites( ) {
    QString s;
    cout << " in ScanSiteWindow::scanSites( ) " << endl;

    recipes_at_atime=gContext->GetSetting("RecipeGetNumRecipesAtATime",
                                  QString("%1").arg(MINRECIPESATATIME)).toInt();

    if (recipes_at_atime<MINRECIPESATATIME) 
        recipes_at_atime = MINRECIPESATATIME*2;

    s = tr("Scanning site www.cooking.com for courses....\n"
                "\tgetting maximum %1 recipes at a time" )
          .arg(recipes_at_atime );

    pd0 = new MythProgressDialog(s , MAXCOURSES);

    if (pd0) {
        pd0->move(1,1);
        pd0->show();
        pd0->update();
    }

    // san default scanning site
    // others may follow
    scan_cooking_com();

    delete pd0;
}    


/*
 *
 *
*/
void ScanSiteWindow::scan_cooking_com( ) {
    const int total_courses=MAXCOURSES;
    int coursenum;
    int end_coursenum;
    int total_recipes_scanned=0;
    int  pagenum = 1;

    cout << " in ScanSiteWindow::scan_cooking_com( ) " << endl;
    coursenum     = gContext->GetSetting("RecipeCourseStart","1").toInt();
    end_coursenum = gContext->GetSetting("RecipeCourseEnd",
                                        QString("%1").arg(MAXCOURSES)).toInt();
    pagenum       = gContext->GetSetting("RecipePageStart","1").toInt();

    if (pagenum<1) pagenum=1;

    if ((coursenum<1)||(coursenum>total_courses))
        coursenum=1;

    if ((end_coursenum<coursenum)||(end_coursenum>total_courses))
        end_coursenum = total_courses;


    while (coursenum <= end_coursenum) {
        MythProgressDialog *pd1=NULL; 
        QStringList reclist;
        QStringList recipelist;
        QStringList ingredients;
        QStringList directions;
        QStringList extra;           // add info
        QStringList links;           // extra urls
        QStringList referencelist;
        QString     course;
        QString     serving;
        QString     comment;
        QString     extra_comment;
        QString     recipe_image;
        QStringList strlst;
        QString s;
        int  cpage=0, npage=0;
        int  bad_page_cnt = 0;
        bool done = false;
        int  cnt = 0;
        int  cur_pagenum = 1;
        int  exit_code=0;
        QString tmpfile;
        QString url;
         
	cout << "while coursenum="<< coursenum << "  end_coursenum=" << end_coursenum << endl;
         
        if (pd0) 
           pd0->setProgress( coursenum );
         	
        while (!done) {
            LynxFile lf(this, "lynx process");
            // setup next course page to grab
            //
            /*
            tmpfile = QString("/tmp/mythRecipe/coursepages/course-%1-%2.html" )
                             .arg(coursenum)
                             .arg(pagenum );
            */
            tmpfile = "/tmp/mythRecipe-course.dump";
            url = tr("http://www.cooking.com/recipes/browse_ctg_det."
                     "asp?Course=%1&orderkey=Course&page=%2" ) 
                    .arg(coursenum)
                    .arg(pagenum);
            lf.getUrlFile(tmpfile, url);
             

            // process current file and add it 
            // to lists.
            if ( !processCoursePage(tmpfile, course, recipelist,
	                                referencelist,cpage,npage) ){
                cout << tr("\t\tinvalid course page ... course %1 page %2")
	                      .arg(coursenum)
	                      .arg(pagenum)
	                 << endl;
	             
                bad_page_cnt++;
                
                if (bad_page_cnt>2) {
                    exit_code=1;
                    break;
                }
            } else {
                // first check if there were any more 
                // course pages
                if ( (cpage+npage)==0 ) {
                    done=true;
                    exit_code=2;
                    break;
                }
                // check if current page more than 
                // total pages
                if ( (cpage>=npage) ) {
                    done=true;
                    exit_code=3;
                    break;
                }
            }
             
            done = ( (cpage>=npage) || 
                   ((total_recipes_scanned+(cur_pagenum*RECIPESPERPAGE)) >
                                            recipes_at_atime) );
            if (cpage>=npage)
                exit_code=4;

            if ( (total_recipes_scanned+(cur_pagenum*RECIPESPERPAGE)) >
                                                            recipes_at_atime )
                exit_code=5;
             
            s = tr("Scanning course number \n\t( %1 - %2 )" ) 
                  .arg(course)
                  .arg( coursenum );

            cout << s.data() << endl;
             
            if (cur_pagenum==1) {
                pd1 = new MythProgressDialog(s, npage );
            }
             
            if (pd1)
                pd1->setProgress(cpage - 19);

            pagenum++;
            cur_pagenum++;
        }

        cout << tr("done getting pages for now (%1)").arg(exit_code) 
             << tr("total recipes scanned (%1)").arg(total_recipes_scanned)
             << endl;

        if (pd1)
            delete pd1;

        s = tr("Processing Recipes from  course \n\t %1 " )
              .arg(course);

        cout << s.data() << endl;

        cpage=0;
        
        pd1 = new MythProgressDialog(s, recipelist.size() );
        cnt=0;  
        
        for ( QStringList::Iterator  it = recipelist.begin(); 
                                     it != recipelist.end(); ++it ) {
            LynxFile lf(this, "lynx process");
            cnt++;
             
            total_recipes_scanned++;
              
            if (pd1) 
                pd1->setProgress(++cpage); 

            reclist = QStringList::split(RECIPESEPARATOR, (*it)); 
            
            /*             
            tmpfile = QString("/tmp/mythRecipe/recipepages/%1" )
                              .arg(QUrl(reclist[0]).fileName()); 
            */
            // tmpfile.sprintf("/tmp/mythRecipe-recipe-%i-%i.dump",
            //                  coursenum, pagenum);
                             
            // tmpfile=QString("/tmp/mythRecipe-recipe-%1-%2.dump")               
            tmpfile = QString("/tmp/mythRecipe-recipe.dump")
                             // .arg(coursenum)
                             // .arg(cnt)
                             ;

            if ( (total_recipes_scanned>recipes_at_atime) ) {
                gContext->SaveSetting("RecipeCourseStart", coursenum);
                gContext->SaveSetting("RecipePageStart", pagenum-1);
                gContext->SaveSetting("RecipeCourseEnd", end_coursenum);
                
                MythPopupBox::showOkPopup(this->parent,
                     QObject::tr("Done getting Recipes for now."),
                     QObject::tr("Done getting Recipes for now "
                                 "There may be more recipes just choose \n"
                                 "Scan Recipe \n menu option again ... "));
                done=true;
                coursenum = end_coursenum;
                break;
            }
           
            if ( (reclist[0].contains(QRegExp(".gif|.jpg"))) || 
                !lf.getUrlFile( tmpfile, reclist[0]) ) {
                cout << tr("error getting recipe page (%1)\n\t(%2)\n\t(%3)")
                           .arg(reclist[0])
                           .arg(tmpfile)
                           .arg(reclist[2])
                     << endl;
            } else {
                recipe_image="";
                if (!processRecipePage(tmpfile,  
                                       serving,
                                       comment,
                                       extra_comment,
                                       recipe_image,
                                       ingredients,
                                       directions,
                                       extra, links ) ) {
                    cout << tr("\terror processing file (%1)")
                              .arg(tmpfile)
                         << endl;
                } else {
                    QByteArray rec_img;
                    // QString img_name = "";
                    QString recipe_img_name = recipe_image.section("/", -1);
                    
                    QString img_name = tr(PREFIX 
                                           "/share/mythtv/themes/default/recipes/%1") 
                                          .arg(recipe_img_name);
                                
                    QFile file( img_name );
                    QDir dir;

                    cout << tr("Recipe image is (%1)").arg(recipe_image) <<endl;
                    
                    if ( (recipe_image != "noimage.jpg") && 
                           lf.getUrlFile( tmpfile, recipe_image, false) ) {
                        if (!file.open( IO_ReadOnly )) {
                            cout << tr("unable to open image file...(%1)")
                                      .arg(img_name) 
                                 << endl;
                        } else {
                            rec_img = file.readAll();
                            file.close();
                            cout << "\tread all data for image of this recipe Closed file= "
                                 << img_name
                                 << endl;
                        }
                    } else 
                        cout << tr("\tno image for this recipe ") 
                             << endl;
                           
                    addRecipeToDatabase( reclist,
                                         course,
                                         serving,
                                         comment,
                                         extra_comment,
                                         recipe_img_name,
                                         rec_img,
                                         ingredients,
                                         directions);
                }
            }
            
        }

        if (pd1)
            delete pd1;
        pd1 = NULL;
        coursenum++;
        pagenum=1;
    }
    if (pd0) {
        pd0->Close();
        // delete pd0;
    }
}

/*
*/


/*
 *
 *
*/
void ScanSiteWindow::addRecipeToDatabase( QStringList &reclist,
                                          QString     &course,
                                          QString     &serving,
                                          QString     &comment,
                                          QString     &extra_comment,
                                          QString     &recipe_img_name,
                                          QByteArray  &recipe_image,
                                          QStringList &ingredients,
                                          QStringList &directions ) { 
    QString update;
    QString select;
    QString insert;
    QSqlQuery query;
    QStringList ing_line;
    int     courseid;
    int     cuisineid;
    int     recipeid;
    int     cnt;


    if (!db) {
        cout << "\tdatabase not available..."
             << endl;
        return;
    }

    // check if course is in database tables

    select = QString("SELECT category_id FROM categories  WHERE "
                     "name = \"%1\" AND parent = \"0\" " )
                     .arg( course ) ;

	
    // query = db->exec(select);
    db->prepare(select);
    db->exec();
    
    if (db->next()) {
        courseid = db->value(0).toInt(); 
    } else {

        insert = QString("INSERT INTO categories (parent, name) "
                         "values(0, \"%1\") ;" )
                         .arg( course );

        // query = db->exec(insert);
        db->prepare(insert);
        db->exec();

        select = QString("SELECT category_id FROM categories  WHERE "
                         "name = \"%1\" AND parent = \"0\" " )
                         .arg( course ) ;

        //query = db->exec(select);
        db->prepare(select);
        db->exec();
	 
        if (!db->next()) {
            cout << tr("\terror trying to add course to database")
                 << endl;
            return;
        }

        courseid = db->value(0).toInt(); 
    }


    select = QString("SELECT category_id FROM categories  WHERE "
                     "name = \"%1\" AND parent = %2 ") 
                     .arg(reclist[3])
                     .arg(courseid );

    // query = db->exec(select);
    db->prepare(select);
    db->exec();


    if ( !db->next() ) {
        insert = QString("INSERT INTO categories (parent, name) "
                         "values(%1, \"%2\") ;")
                       .arg(courseid)
                       .arg(reclist[3]);

        // query = db->exec(insert);
        db->prepare(insert);
        db->exec();

        select = QString("SELECT category_id FROM categories  WHERE "
                         "name = \"%1\" AND parent = %2 " ) 
                       .arg(reclist[3])
                       .arg(courseid );

        // query = db->exec(select);
        db->prepare(select);
        db->exec();
        
        if (!db->next()) {
            cout << tr("\terror trying to add cuisine to database")
                 << endl;
            return;
        }
        
        cuisineid = db->value(0).toInt();

    } else 
        cuisineid = db->value(0).toInt(); 

    select = ("SELECT recipe_id FROM recipes WHERE name=\""+reclist[1]+"\" ");

    // query = db->exec(select);
    db->prepare(select);
    db->exec();

    if ( db->next() ) {
        // did not find recipe...
        return;
    } else {
    /*
        insert = QString("INSERT INTO recipes "
                      "(name, "
                      "yield_qty, "
                      "yield_units, "
                      "source, "
                      "author_user_id,"
                      "create_utime,"
                      "mod_utime, "
                      "comment, "
                      "extra_comment, "
                      "image_name, "
                      "recipe_image ) "
                      "values( '%1', '%2', '%3', '%4', '%5', '%6', '%7', '%8', '%9', '%10' ) ")
                   .arg(reclist[1])
                   .arg(0 )
                   .arg(serving)
                   .arg(reclist[2]) 
                   .arg(2 )
                   .arg(QDateTime::currentDateTime().toString())
                   .arg(QDateTime::currentDateTime().toString())
                   .arg( comment)
                   .arg( extra_comment)
                   .arg( recipe_img_name);
        db->prepare(insert);
     */
        db->prepare("INSERT INTO recipes "
                      "(name, "
                      "yield_qty, "
                      "yield_units, "
                      "source, "
                      "author_user_id,"
                      "create_utime,"
                      "mod_utime, "
                      "comment, "
                      "extra_comment, "
                      "image_name, "
                      "recipe_image ) "
                      "values( ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ? ) ");
        db->addBindValue( reclist[1]); 
        db->addBindValue( 0 ); 
        db->addBindValue( serving); 
        db->addBindValue( reclist[2]); 
        db->addBindValue( 2 ); 
        db->addBindValue( QDateTime::currentDateTime().toString());
        db->addBindValue( QDateTime::currentDateTime().toString());
        db->addBindValue( comment);
        db->addBindValue( extra_comment);
        db->addBindValue( recipe_img_name);
        db->addBindValue( recipe_image );

        if (!db->exec()) 
            cout << tr("could not add recipe successfully ...") << endl;
        	
        if (db->lastError().type() != QSqlError::None) {
            QString msg =
            QString("Query was: %1 \nError was: %2 \n(%3)")
            .arg(db->lastQuery())
            .arg(MythContext::DBErrorMessage(db->lastError()))
            .arg(comment);
            VERBOSE(VB_IMPORTANT, msg);
            return;
        }
        	
        	
        select = ("SELECT recipe_id FROM recipes WHERE name=\""+reclist[1]+"\"");

        db->prepare(select);
        db->exec();
        
        if (!db->next()) {
            cout << tr("\terror trying to add recipe to database")
                 // << tr(insert)
                 << endl;
            return;
        }
        
        recipeid = db->value(0).toInt();
    }


    insert = QString("INSERT INTO category_entries (recipe_id, category_id) "
                     " values(%1, %2) ;")
                     .arg(recipeid)
                     .arg(cuisineid);

    db->prepare(insert);
    db->exec();

    // #instructions are here....
    cnt=0;
    for ( QStringList::Iterator it=directions.begin();
                               it!=directions.end(); ++it ) {
        insert = QString("INSERT INTO instructions values(%1, %2, \"%3\")")
                          .arg(recipeid)
                          .arg(cnt++)
                          .arg( (*it) );

        db->prepare(insert);
        db->exec();
    }
    /*
    // # add references to pictures to the instructions for now
    insert.sprintf("INSERT INTO instructions values(%i,3,\"references\") ",
                     recipeid );
    query = db->exec(insert);

    */

    cnt=0;
    for ( QStringList::Iterator it=ingredients.begin();
		    it!=ingredients.end(); ++it ) {
        ing_line = QStringList::split("|",  (*it) );
        insert = QString("INSERT INTO ingredients " 
               "(recipe_id,display_order,ingredient_type,name, qty, unit_id) "
         "values(%1, %2, %3, \"%4\", %5, %6 ) " )
        .arg(recipeid) 
        .arg(cnt++) 
        .arg(ing_line[0].toInt()) 
        .arg(ing_line[3]) 
        .arg(ing_line[1].toFloat()) 
        .arg(ing_line[2].toInt() );

        db->prepare(insert);
        db->exec();
    }

}



/*
 *
 *
*/
void ScanSiteWindow::enterPressed( ) {

    scanSites();
}


/*
 *
 *
*/
bool ScanSiteWindow::processCoursePage( QString     &lynx_file_name, 
                                        QString     &course,
                                        QStringList &recipelist,
                                        QStringList &referencelist,
                                        int         &current_page,
                                        int         &total_page    ) {
    QString line=0;              // current line from page
    QString location=0;          // location in page title
    QString cuisine=0;           // cuisine at top of page
    QString s;
    QString lastline;
    QString pcnt;                // page count found on  page
    QStringList pagecnt;         // page count found on  page
    QRegExp rx;                  // search away ...

    cout << "in canSiteWindow::processCoursePage" << endl;
    referencelist = QStringList::split(" ","");

    // cout << "ScanSiteWindow::processCoursePage() "  << endl;

    QFile *lynx_file = new QFile(lynx_file_name);

    if (!lynx_file) {
        cout << tr("error memory low...?") << endl;
        return false;
    }

    if (!lynx_file->open( IO_ReadOnly )) {
        cout << tr("unable to open local recipe file...") 
             << lynx_file_name.data() << endl;
        delete lynx_file;
        return false;
    } 

    nextLine( lynx_file );

    // line = nextLine( lynx_file );
    location = 
        nextLine(lynx_file).section(":",1,1).section("-",0,0).stripWhiteSpace();

    line = nextLine( lynx_file, QRegExp("Recipe Browse >"), lastline);
    course = lastline.section(">", 1, 1).stripWhiteSpace();

    if ( lynx_file->atEnd() || course.isEmpty() || course=="") {
        cout << tr("problem with finding Recipe Browse in file... "
                   "file bad ??? ")
             << endl;
        delete lynx_file;
        return false;
    } 

    // cout << "\tlocation (" << location.data() <<") ... ("
    //      << course.data() << ")" << endl;
    line = nextLine( lynx_file, QRegExp("ecipes "), lastline);

    if (lastline.contains("no recipes were")>0) {
        cout << "\tsorry no more recipes were found..." << endl;
        current_page=total_page=0;
        delete lynx_file;
        return true;
    }
    rx.setPattern("(\\d*) - (\\d*)\\) of (\\d*)");
    // rx.search(line.section("\n",-2,-2));
    rx.search(lastline);

    if (rx.cap(0).isEmpty() || (rx.cap(1)=="") ) {
        cout << tr("problem with finding start of page "
                   "count in file \n\t\tfile bad ??? (%1") 
                   .arg(lastline.data())
             << endl;
        delete lynx_file;
        return false;
    } 
    /*
    cout << "\tpage count ("
         << rx.cap(0).toInt()
         << ")  ("
         << rx.cap(1).toInt()
         << ")  ("
         << rx.cap(2).toInt()
         << ")  ("
         << rx.cap(3).toInt()
         << ")  "
         << endl;
    */
    current_page = rx.cap(1).toInt();
    total_page   = rx.cap(3).toInt();

    if (rx.cap(1).toInt() > rx.cap(3).toInt()) {
            cout << tr("page count too large we must be all done ...") <<endl;
        current_page = rx.cap(1).toInt();
        delete lynx_file;
        return true;
    } 

    // throw away the next two lines
    nextLine( lynx_file );
    nextLine( lynx_file );

    line = nextLine( lynx_file, QRegExp("(\\d*) - (\\d*)\\) of (\\d*)") );

    s = nextLine( lynx_file, QRegExp( "References" ) );
    if ( lynx_file->atEnd() || s.isEmpty() ) {
        cout << tr("could not find References...") << endl;
        delete lynx_file;
        return false;
    } else {
        // skip next line
        // should be blank.
        //
        nextLine( lynx_file );
        /*
        s = nextLine(lynx_file,QRegExp("mixlipix"));
        referencelist = QStringList::split("\n", s);

        rx.setPattern("(\\d*) - (\\d*)\\) of (\\d*)");
        rx.search(line.section("\n",-2,-2));

        if ( rx.cap(0).isEmpty() || rx.cap(1)=="") {
            cout << "\t\tproblem with finding end of " 
                 << "course and recipe list in file \n"
                 << "\t\tfile bad ??? (" 
                 << line.section("\n",-2,-2).data()
                 << ") " << endl;
            return false;
        } 
        */                
        scanCourseRecipe( lynx_file, course, line, recipelist);
    }
    delete lynx_file;
    return true;
}



/*
 *
 *
*/
void ScanSiteWindow::scanCourseRecipe( QFile *file, QString course, 
                                       QString buffer, 
                                       QStringList &retstr ) {
    // QStringList retstr;
    QString cuisine=0;
    QString recipename=0;
    QString source=0;
    QString link=0;
    QString photo=0;
    QString line=0;
    QRegExp rx;

    QStringList strlist =QStringList::split("\n",  buffer);

    cout << "ScanSiteWindow::scanCourseRecipe() "  << endl;

    for ( QStringList::Iterator it = strlist.begin(); 
                                it != strlist.end(); ++it ) {
        QStringList record;
        QString s = (*it);
            
        if ( s.contains(" Cuisine ") )
            it++;
   
            s = (*it);

            photo = ((s.contains("photo avail")) ? ("pa") : ("np") );

            rx.setPattern("\\d+");
            rx.search( s.mid(17) );

            if (rx.cap(0).isEmpty() || rx.cap(0)=="")
                break;
             
            link = rx.cap(0)+".";
             
            nextLine(file, QRegExp(link.data()), line);
	    if (!file->atEnd()) {
                link = line.section(". ", 1, 1);
            } else {
                link = NOLINKFOUND;
            }
             
            recipename = (s.replace(QRegExp("\\d*||transp.gif||\[]\[]|photo available"), "")).stripWhiteSpace();
            // recipename = s.replace(QRegExp("photo available"), "");
            // recipename = recipename.stripWhiteSpace();
            if ( (recipename.isEmpty()) || (recipename.isNull()) ) {
                break;
            } 

            it++;
            s = (*it);

	    if ( s.contains( "Source:" ) ) {
                source  = s.replace(QRegExp("\\d*||transp.gif||\[]\[]|Source:"), "");
	    } else
	        it--;

            it++;
            s = (*it);
            cuisine  = s.replace(QRegExp("\\d*||transp.gif||\[]\[]"), "");

            record.append(link.stripWhiteSpace());
            record.append(recipename.stripWhiteSpace());
            record.append(source.stripWhiteSpace());
            record.append(cuisine.stripWhiteSpace());
            record.append(photo.stripWhiteSpace());
            record.append(course.stripWhiteSpace());

            retstr.append(record.join("~~"));
    }
}



/*
 *
 *
*/
QString ScanSiteWindow::nextLine( QFile *file ) {
    QString str=0;
    Q_ULONG len=100000;
    Q_LONG res;

    if (file) 
        if (! file->atEnd() )
            res = file->readLine( str, len );
    return str;
}


/*
 *
 *
*/
QString ScanSiteWindow::nextLine( QFile *file, QRegExp rx, QString &line ) {
    QString str=0;
    // QString line=0;
    int len=100000;
    Q_LONG res;
    int done=0;

    line = "";

    if (file) 
        while ( (!file->atEnd()) && !done ) {
            res = file->readLine( line, len );
            if (!(done = (line.contains( rx )>0)))
                str += line;
        }
    return str;
}


/*
 *
 *
*/
QString ScanSiteWindow::nextLine( QFile *file, QRegExp rx ) {
    QString str=0;
    QString line=0;
    int len=100000;
    Q_LONG res;
    int done=0;

    if (file) 
        if (rx.isEmpty() || !rx.isValid())  {
            res = file->readLine( str, len );
        } else {
            while ( (!file->atEnd()) && !done ) {
                res = file->readLine( line, len );
                done = (line.contains( rx )>0); 
                str += line;
            }
        }
    return str;
}


/*
*/


/*
 *
 *
*/
ScanSiteWindow::~ScanSiteWindow(void) {
	
}


/*
 *
 *
*/
void ScanSiteWindow::keyPressEvent(QKeyEvent *e ) {
    bool handled = false;
    QStringList actions;
    gContext->GetMainWindow()->TranslateKeyPress("Recipe", e, actions);

    // cout << "ScanSiteWindow::keyPressEvent() "
    // 	 << endl;

    for (unsigned int i = 0; i < actions.size() && !handled; i++) {
        QString action = actions[i];
        handled = true;

	cout <<"\t\t"<<action.data() << endl;

        if (action == "SELECT") {
	    enterPressed();
	}
        else
            handled = false;
    }

    if (!handled)
        MythThemedDialog::keyPressEvent(e);
    
    updateForeground();
}



/*
 *
 *
*/
bool ScanSiteWindow::processRecipePage( QString &lynx_file_name,
                                        // QStringList &rec_info,
                                        QString     &serving,
                                        QString     &comment,
                                        QString     &extra_comment,
                                        QString     &recipe_image,
                                        QStringList &ingredients,
                                        QStringList &directions,
                                        QStringList &extra,       // add info
                                        QStringList &references){ // extra urls
    QString      line; 
    QString      trash; 
    QString      recipe_image_link;     // link to image
    QString      page_title;
    QString      source;
    QStringList  strlst;
    QString      s1,s2,s3;
    QRegExp      rx;
    QStringList::Iterator it;
    QFile       *lynx_file = new QFile(lynx_file_name);

    extra = references;
    references = extra;
    // rec_info = QStringList::split(" ","");

    // cout << "ScanSiteWindow::processRecipePage() "  << endl;
    cout << tr("working on recipe page (filename : %1)")
     	      .arg(lynx_file_name.data())
     	 << endl;

    if (!lynx_file) {
        cout << tr("error memory low !?") << endl;
        return false;
    }

    if (!lynx_file->open( IO_ReadOnly )) {
        cout << tr("unable to open recipe file %1")
                  .arg(lynx_file_name.data())
             <<endl;
        delete lynx_file;
        return false;
    } 

    // skip THE_URL
    nextLine(lynx_file);
    
    nextLine(lynx_file, QRegExp("THE_TITLE:"), line );
    if ( lynx_file->atEnd() || line.isEmpty() || (line=="") ) {
        cout << tr("cannot find The page title boo hoo..")
             << endl;
        delete lynx_file;
        return false;
    }

    // page_title=QStringList::split(" - ",(QStringList::split(":",line))[1]);
    // page_title = line.section(":",1,1).section(1,1);
    page_title = line.section("-", 1, 1).stripWhiteSpace();

    nextLine(lynx_file, QRegExp("redotlin333"), trash);

    s1 = (nextLine( lynx_file )).simplifyWhiteSpace();
    // s2 = page_title[1].simplifyWhiteSpace();
    s1 = (nextLine( lynx_file )).simplifyWhiteSpace();
    // s2 = page_title.simplifyWhiteSpace();
    
    if (s1!=page_title) {
        page_title = s1;
    }
    
    s1 = (nextLine( lynx_file )).simplifyWhiteSpace();

    if (s1.contains("Source:")) {
        source = QStringList::split("Source:",s1,trash)[1];
        cout << QString("source was: %1.\n").arg(source); 
        
        s1 = (nextLine( lynx_file )).simplifyWhiteSpace();
    }
     
   
    strlst = QStringList::split("\n", 
                               nextLine(lynx_file,QRegExp("rerecite"), trash));
    // strlst.pop_back();

    serving = "";
    for ( it=strlst.begin(); it!=strlst.end(); it++ ) {
        serving += ((*it).stripWhiteSpace()+"\n");
    }
    serving = serving.stripWhiteSpace();

    if ( serving.isEmpty() || serving=="" ) {
	serving = ".";
    } 

    // cout << tr("serving was: %1.\n").arg(serving);
        
    // line = nextLine(lynx_file, QRegExp("rereciteq03b"));
    nextLine(lynx_file, QRegExp("rereciteq03b"), trash);
    
    if ( lynx_file->atEnd() ) {
        delete lynx_file;
        return false;
    }

    nextLine( lynx_file );

    line = (nextLine(lynx_file, QRegExp("trans"), trash )).stripWhiteSpace();

    if (lynx_file->atEnd() ) {
        cout << tr("no comment found... problem !") << endl;
        delete lynx_file;
        return false;
    } 

    rx.setPattern("RE_\\w+.jpg");
    rx.search(line);
	
    recipe_image = rx.cap(0);
    if (recipe_image.isEmpty() || recipe_image=="") {
        // recipe_image = "9999. noimage.jpg";
        recipe_image = "noimage.jpg";
    } else {
        rx.setPattern("\\d+");
        rx.search(line);
     
        recipe_image_link = rx.cap(0) + ". ";
        // recipe_image.sprintf("%s. %s", rx.cap(0).data(), recipe_image.data() ); 
    }
    // cout << QString(" image name %1.\nimage link: %2\n").arg(recipe_image)
    //                                                    .arg(recipe_image_link);

    // remove image name and link from line
    // comment = line;
    comment = ((line.replace(
            QRegExp("RE_\\w+.jpg||\\d*||transp.gif||\[]\[]"), "")
				).stripWhiteSpace()) +".";
/*
    line = nextLine( lynx_file );

cout << QString("comment: %1.\n").arg(comment);
cout << QString("next line : %1.\n").arg(line);

    if (line.contains("rereciteq01")) {  // end of comment section go on
        s1="";                           // to read ingredients....
    } else if (line.contains("RECIPE INGREDIENTS")) {
        s1="";
    } else {
        s1 = line + nextLine( lynx_file, QRegExp("rereciteq01"), trash);
	// strlst = QStringList::split("\n", s1);
	// strlst.pop_back();
        // s1 = strlst.join("\n");
    }
    extra_comment = s1.stripWhiteSpace()+".";

    // s1 = nextLine( lynx_file, QRegExp("rereciteq01"), trash);
*/    
    s1 = nextLine( lynx_file, QRegExp("RECIPE METHOD"), trash);

    if ( lynx_file->atEnd() ) {
        cout << tr("error finding ingredients marker !"
                   "Ingredient ?: %1.").arg(s1) 
             << endl;
        delete lynx_file;
        return false;
    } 

    // strlst = QStringList::split("\n", s1);
    // strlst.pop_back();
    // s1 = strlst.join("\n");

    ingredients = decipherIngredients( s1 );
    /*
    */

    // s1 = nextLine( lynx_file, QRegExp("transp"), trash);
    s1 = nextLine( lynx_file, QRegExp("transp.gif"), trash);
    directions = QStringList::split("\n", s1);
    // directions.pop_back();
    // cout << tr("Directions : %1.\n").arg(s1) << endl;

    nextLine( lynx_file, QRegExp("Reference"), trash);
    nextLine( lynx_file );

    if ( lynx_file->atEnd() ) {
        cout << tr("error finding Reference marker !")
             << endl;
        delete lynx_file;
        return false;
    } 

    // references=QStringList::split("\n",nextLine(lynx_file,QRegExp("mxlpix")));
    //

    
    if (recipe_image != "noimage.jpg" ) {
        line="";
        nextLine(lynx_file, QRegExp(recipe_image), line);
        if ( !(line.isEmpty()||line.isNull()) ) {
            recipe_image = line.section(". ", 1).stripWhiteSpace();
        } else
            recipe_image = "noimage.jpg" ;
    }

    delete lynx_file;
    return true;
}


/*
 *
 *
*/
QStringList ScanSiteWindow::decipherIngredients( QString line ) {
    QString     select;
    QSqlQuery   query;
    QStringList retlist;
    QStringList record;
    QStringList ing_list;
    QStringList ing_line;
    QStringList number_list;
    QStringList link_list;
    QRegExp     rx;
    QString     ing=0;
    int         unitid=0;
    double      quantity=0;
    QString     item=0;
    int         ingtype=0;
    int         cnt=0;

    QString str;
    QString str1;

    ing_list = QStringList::split("\n", line);
    ing_list.pop_back();

    for ( QStringList::Iterator it=ing_list.begin();
		    it!=ing_list.end(); ++it ) {
        item = (*it).stripWhiteSpace();
        ing_line = QStringList::split(" ", item );
	ingtype=0;
	unitid=0;
	quantity=0;
        if ((item).contains(QRegExp("^For "))) { 
            ingtype=2;
        } else {
            rx.setPattern("^\\d+ \\d+/\\d+");
            rx.search(item);
            if ( !(rx.cap(0).isEmpty() ||rx.cap(0)=="") ) {
                number_list = QStringList::split( QRegExp(" |/"), rx.cap(0) ); 
                if (number_list[2].toInt()==0) {       
                    quantity=number_list[0].toInt();
                } else
                    quantity = 
                       number_list[0].toFloat()+
                            (number_list[1].toFloat()/number_list[2].toFloat());
                ing_line.pop_front();
                ing_line.pop_front();
                item = ing_line.join(" ");
            } else { 
                rx.setPattern("^\\d+/\\d+");
                rx.search(item);
                if ( !(rx.cap(0).isEmpty() ||rx.cap(0)=="") ) {
                    number_list=QStringList::split(QRegExp(" |/"), rx.cap(0) ); 
                    if (number_list[1].toInt()==0) {
                        quantity=number_list[0].toFloat();
                    } else
		       quantity=(number_list[0].toFloat()/number_list[1].toFloat());
                       ing_line.pop_front();
                       item = ing_line.join(" ");
                } else {
                    rx.setPattern("^\\d+");
                    rx.search(item);
                    if ( !(rx.cap(0).isEmpty()||rx.cap(0)==""||rx.cap(0).toInt()==0) ){
                        quantity = rx.cap(0).toFloat();
                        ing_line.pop_front();
                        item = ing_line.join(" ");
                    } else {
                    }
                }
	    }
            
            // check ingredient unit type
	    //
            select = "SELECT unit_id FROM units WHERE name LIKE \"\%"+
                              ing_line[0] + "\%\" ";
            // query = db->exec(select);
            db->prepare(select);
            db->exec();
            
	    if (db->next()) {
                unitid = db->value(0).toInt();
                select = QString("SELECT unit_id FROM units WHERE name=\"%1\"")
                                 .arg( ing_line[0]+ing_line[1] );
                // query = db->exec(select);
                db->prepare(select);
                db->exec();
                
	        if (db->next()) {
                    unitid = db->value(0).toInt();
                    ing_line.pop_front();
                    ing_line.pop_front();
                } else {
                    ing_line.pop_front();
                }
                item = ing_line.join(" ");
            } else {
            }
        }


        rx.setPattern("\\d*");
        rx.search(item);
 
	cnt=0;
        for (cnt=0;cnt<rx.numCaptures();cnt++) {
            link_list.append(rx.cap(cnt++));
        }

        if (cnt>0) {
            cout << "\t(" << cnt << ") links found..("
                 << link_list.join("/").data()
                 << ") " << endl;
        }

        // item = item.replace(QRegExp("\\d*||\[]\[]"), "");
        // item = item.replace(QRegExp("\\d*||transp.gif||\[]\[]"), "");

	str = QString("%1|%2|%3|%4")
                      .arg(ingtype)
                      .arg(quantity)
                      .arg(unitid)
                      .arg(item);

        retlist.append(str);	
    }
    return retlist;
}



/*
 *
 *
*/
void ScanSiteWindow::wireUpTheme() {

}
