#include <mythtv/mythcontext.h>
#include <mythtv/mythwidgets.h>
#include <mythtv/lcddevice.h>
#include <mythtv/xmlparse.h>

#include <qurl.h>
#include <qlabel.h>

#include "ScanXML.h"
#include "DisplayStatus.h"

#include "GetLynxFile.h"

#include "menu.h"
#include "ParseML.h"

#define NOLINKFOUND      "@"
#define RECIPESEPARATOR  "~~"

#define RECIPEEXISTS   00001


/*
 *
 *
*/
ScanXML::ScanXML(MythMainWindow *parent, MSqlQuery *ldb, 
               QString window_name, QString theme_filename, const char *name)
           : MythThemedDialog(parent, window_name, theme_filename, name)
{
    QDir dir;
    QDir tmpdir;
    Menu menu;
    ParseML parsexml;
    QString xmlpath = gContext->GetSetting("RecipeLoadLocation");
    QString s_dir = PREFIX "/share/mythtv/themes/default/recipes/.";
    QString s="";
    MythProgressDialog *pd1;

    cout << "ScanXML::ScanXML()...." << endl;

    //  A few internal variable defaults
    if ( (db=ldb)) {
        recipedb = new RecipeDB(this, "recipedb", db);        
    } else {
	 MythPopupBox::showOkPopup(parent,
              QObject::tr("Database not available."),
              QObject::tr("This module option will be implemented soon... "));
        return;
    }

    if ( xmlpath.isEmpty() || xmlpath.isNull() ) {
        xmlpath = PREFIX "/share/mythtv/mythRecipe/xml";
    }


    dir.setPath(xmlpath);
    dir.setFilter( QDir::Files | QDir::Hidden | QDir::NoSymLinks );
    dir.setSorting( QDir::Size | QDir::Reversed );
    dir.setNameFilter("*.xml");

    if (!dir.exists() ) {
        if (!dir.mkdir(xmlpath)) {
            MythPopupBox::showOkPopup(parent,
                  QObject::tr("Your directory " + xmlpath + "does not exist"),
                  QObject::tr("Your directory " + xmlpath + "does not exist"));
            return;
	}
    }

    if ( !dir.setCurrent( xmlpath ) ) {
        MythPopupBox::showOkPopup(parent,
        QObject::tr("Error changing to Your directory. "+
		xmlpath+". Does not exist"),
        QObject::tr("Error changing to Your directory. "+
		xmlpath+". Does not exist"));
        return;
    }

    s = ("Importing XML files found in " + xmlpath);
    pd1  = new MythProgressDialog(s, dir.count() );


    for (unsigned int i=0; i<dir.count();i++) {
        QString newname = (dir[i] + ".imported");

        // parseml.ParseMLFile("myrecipe.xml", menu);
        parsexml.ParseMLFile(dir[i].latin1(), menu);

        if (pd1)
            pd1->setProgress( i );

        dir.rename(dir[i], newname);
        //cout <<"ScanXML::ScanXML()... done backing up ... " << newname <<endl;
    }

    if (pd1)
        delete pd1;

    addRecipeToDatabase( menu );

    MythPopupBox::showOkPopup(parent,
              QObject::tr("Done Importing Recipes."),
              QObject::tr("Done Importing Recipes... "));
    return;
}



/*
 *
 *
*/
void ScanXML::importXML( QString &filepath) {

}



/*
 *
 *
*/
int ScanXML::addRecipeToDatabase( Recipe &recipe ) {
    Header head = recipe.getHeader();
    QString recipename = head.getTitle();
    QString yield = head.getYield().value();
    QValueList<QString> &categories = head.getCategory();
    QValueList<Ingredient> &ingredients = recipe.getIngredient();
    QValueList<QString> &directions = recipe.getDirection();
     
    int recipeid = 0;
    int courseid = 0;
    int cnt=0;


    if ( (recipeid=addRecipe( head )) <0 ) {
        switch (recipeid) {
            case -2 :
                     cout << "(-1) unable to add recipe" << endl;
                     break;
            case -1 :
                     cout << "(-2) unable to add recipe" << endl;
                     break;
        }
        return recipeid;
    }

    // cout << "processing Categories in recipe " << endl;
    for (QValueList<QString>::Iterator it=categories.begin();
                                      it!=categories.end();++it ) {
        if ( (courseid=addCourse((*it), 1))>=0 ) {
            if ( (courseid=addRecipeToCourse(recipeid, courseid))>=0 ) {
            } else {
               cout << "unable to add recipe to course " << (*it) << endl;
            }
        } else {
           cout << "unable to add course " << (*it) << endl;
        }
    }

    // int cnt=1;
    // cout << "processing Ingredients in recipe " << endl;
    for (QValueList<Ingredient>::Iterator it=ingredients.begin();
                                          it!=ingredients.end();++it ) {
        addIngredients(recipeid, (*it));
    }

    // cout << "processing Directions in recipe " << endl;
    cnt=0;
    for (QValueList<QString>::Iterator it=directions.begin();
                                      it!=directions.end();++it ) {
        QStringList sl = QStringList::split(". ", (*it));
        for (QStringList::Iterator line=sl.begin();
                                  line!=sl.end(); ++line ) {
            addInstructionLine(recipeid, cnt++, (*line));
        }
    }
    
    return recipeid;
}


/*
 *
 *
*/
int ScanXML::addInstructionLine( int recipeid, int linenumber, QString &step ) {
    QString insert;
    QSqlQuery query;

    //
    // #instructions are here....
    insert = QString("INSERT INTO instructions values(%1,%2,\"%3\")")
                                                       .arg(recipeid)
                                                       .arg(linenumber)
                                                       .arg(step);
    // query = db->exec(insert);
    db->prepare(insert);
    db->exec();

    return 0;
}


/*
 *
 *
*/
QString ScanXML::getAuthorName( int authorid ) {
    QString select;
    QSqlQuery query;

    select.sprintf("SELECT user_name FROM users  WHERE "
              "user_id = %d ", authorid );

    // query = db->exec(select);
    db->prepare(select);
    db->exec();
    
    if (db->next()) {
        QString str = db->value(0).toString(); 
        if (str.isEmpty() || str.isNull() ) {
            return "unKnown";
        } else 
            return str;
    } else 
        return "unKnown";
}


/*
 *
 *
*/
int ScanXML::getAuthorId( QString &author, bool addifnotexist ) {
    QString select;
    QString insert;
    QSqlQuery query;

    if ( author.isEmpty() ||author.isNull())
        return 0;

    select = ("SELECT user_id FROM users  WHERE "
              "user_name = \"" + author + "\" " );

    // query = db->exec(select);
    db->prepare(select);
    db->exec();
    
    if (db->next()) {
        return  db->value(0).toInt(); 
    } else {
        if (addifnotexist) {

            insert = "INSERT INTO users ( user_name) "
                     "values(0, \"" + author + "\") ;";
    
            // query = db->exec(insert);
            db->prepare(insert);
            db->exec();

            select = ("SELECT user_id FROM users  WHERE "
                      "user_name = \"" + author + "\" " );
    
            // query = db->exec(select);
            db->prepare(select);
            db->exec();
    	 
            if (!db->next()) {
                cout << "\terror trying to add author to database"
                     << endl;
	        return 0;
            } else
                return db->value(0).toInt(); 
        } else
            return 0; 
    }
    return 0;
}


/*
 *
 *
*/
int ScanXML::addIngredients(int recipeid, Ingredient &ingredients ) {
    int cnt=0;
    QSqlQuery query;
    QString insert;

    QString &title = ingredients.getTitle();
    QString &descr = ingredients.getDescription();
    QValueList<IngItem> &ing_items = ingredients.getIngItems();

    if (!(title.isEmpty()||title.isNull()|| title=="none") ){
        cout << "Ingredient Title: ";
        cout << title;
        cout << endl;
        insert = QString("INSERT INTO ingredients " 
               "(recipe_id,display_order,ingredient_type,name, qty, unit_id) "
        "values(%1,       %2,           %3,          \"%4\", \"%5\", \"%6\") ")
            .arg(recipeid)
            .arg(cnt++)
            .arg(2)
            .arg(title)
            .arg(0)
            .arg(0);

        // query = db->exec(insert);
        db->prepare(insert);
        db->exec();
    }
    if (!(descr.isEmpty()||descr.isNull()||descr=="none") ) {
        cout << "Description : ";
        cout << descr;
        cout << endl;
    }

    for ( QValueList<IngItem>::Iterator it=ing_items.begin();
		              it!=ing_items.end(); ++it ) {
        QValueList<Amount> &amt = (*it).getAmount();

        insert = QString("INSERT INTO ingredients " 
               "(recipe_id,display_order,ingredient_type,name, qty, unit_id) "
        "values(%1,       %2,           %3,          \"%4\", \"%5\", \"%6\") ")
            .arg(recipeid)
            .arg(cnt++)
            .arg(1)
            .arg((*it).getItem())
            .arg(amt[0].qty())
            .arg(getUnitId(amt[0].unit()));

        // query = db->exec(insert);
        db->prepare(insert);
        db->exec();
    }
    return 0;
}


/*
 *
 *
*/
int ScanXML::addRecipe( Header &recipehead ) {
    QString insert;
    int authorid=0;
    int recipeid=0;

    authorid = getAuthorId( recipehead.getAuthor() );


    recipeid = getRecipeId(recipehead.getTitle());

    if (recipeid<(-1)) {
        cout << "ScanXML::addRecipe()... invalid recipe name" << endl;
    } else if (recipeid<0) { 
	insert = QString("INSERT INTO recipes (name, yield_qty, yield_units, "
                         "source, author_user_id,create_utime,mod_utime) "
          " values(\"%1\", \"%2\", \"%3\", \"%4\", \"%5\", \"%6\", \"%7\" ) ")
                 .arg(recipehead.getTitle())
                 .arg(recipehead.getYield().qty())
                 .arg(recipehead.getYield().unit())
                 .arg(recipehead.getSource())
                 .arg(authorid)
                 .arg(QDateTime::currentDateTime().toString())
                 .arg(QDateTime::currentDateTime().toString());
         
        // QSqlQuery query = db->exec(insert);
        db->prepare(insert);
        db->exec();
	
        if ( (recipeid = getRecipeId(recipehead.getTitle()))<0) {
            cout << "ScanXML::addRecipe()... error trying to add recipe to database"
		 << endl;
        }
    } else
        cout << "ScanXML::addRecipe()... found recipe not adding" << endl;

    return recipeid;
}


/*
 *
 *
*/
int ScanXML::addRecipeToCourse( int recipeid, int course ) {
    QString select;
    QString insert;

    select = QString("SELECT category_id FROM category_entries  WHERE "
             "category_id = %1 AND recipe_id = %2 ")
              .arg(course)
              .arg(recipeid);

    // QSqlQuery query = db->exec(select);
    db->prepare(insert);
    db->exec();
            
    if ( !(db->next()) )  {
        insert = QString("INSERT INTO category_entries (category_id, recipe_id) "
                 "values(%1, %2) ;")
                  .arg(course)
                  .arg(recipeid);

        // query = db->exec(insert);
        db->prepare(insert);
        db->exec();

        return course;
    } else
        return db->value(0).toInt(); 
}



/*
 *
 *
*/
int ScanXML::addCourse( QString &course, int parent ) {
    QString select;
    QString insert;
    int courseid;

    if ( course.isEmpty() || course.isNull())
        return -2;
	    
    select = QString("SELECT category_id FROM categories  WHERE "
             "name = \"%1\" AND parent = \"%2\" ")
              .arg(course)
              .arg(parent);

    // QSqlQuery query = db->exec(select);
    db->prepare(select);
    db->exec();
            
    if ( (db->next()) )  {
        courseid =  db->value(0).toInt(); 
    }  else {
        insert = QString("INSERT INTO categories (parent, name) "
                 "values(%1, \"%2\") ;")
                  .arg(parent)
                  .arg(course);

        // query = db->exec(insert);
        db->prepare(insert);
        db->exec();

        select = QString("SELECT category_id FROM categories  WHERE "
             "name = \"%1\" AND parent = %2 ")
              .arg(course)
              .arg(parent);

        // query = db->exec(select);
        db->prepare(select);
        db->exec();
	 
        if (!db->next()) {
            cout << "\terror trying to add course to database"
		 << endl;
	    return -1;
        }

        courseid = db->value(0).toInt(); 

    }
    return courseid;
}


/*
 *
 *
*/
int ScanXML::getUnitId( QString &unit, bool addifnotexist ) {
    QString select;
    QString insert;

    if ( unit.isEmpty() || unit.isNull()) {
        return 0;
    }

    select = ("SELECT unit_id FROM units  WHERE "
		    "name LIKE  \'\%" + unit + "\%\' " ) ;

    // QSqlQuery query = db->exec(select);
    db->prepare(select);
    db->exec();
            
    if (db->next()) {
        return  db->value(0).toInt(); 
    } else {
        if (addifnotexist) {
            insert = ("INSERT INTO units (name, abbreviation) "
			"values(\"" + unit + "\", \"" + unit + "\") ;" );
            // query = db->exec(insert);
            db->prepare(insert);
            db->exec();

            select = ("SELECT unit_id FROM units  WHERE "
                      "name LIKE  \'\%" + unit + "\%\' " ) ;
            cout << "ScanXML::getUnitId()... inserting ("<< unit << ") " ;

            // QSqlQuery query = db->exec(select);
            db->prepare(select);
            db->exec();
            
            if (db->next()) {
                return  db->value(0).toInt(); 
            } else  {
                return 0;
            }
        } else
           return 0; 
    }

}


/*
 *
 *
*/
int ScanXML::getCourseId( QString coursename, int parent ) {
    QString select;


    if ( coursename.isEmpty() ||coursename.isNull())
        return -2;

    select = QString("SELECT category_id FROM categories  WHERE "
		    "name = \"%s\" AND parent = %d ")
                     .arg(coursename)
                     .arg(parent);

    // QSqlQuery query = db->exec(select);
    db->prepare(select);
    db->exec();
            
    if (db->next()) {
        return  db->value(0).toInt(); 
    } else {
        return -1; 
    }
}


/*
 *
 *
*/
int ScanXML::getRecipeId( QString &recipename ) {
    QString select;


    if ( recipename.isEmpty() || recipename.isNull() )
        return -2;
	    
    select = ("SELECT recipe_id FROM recipes WHERE "
		    "name = \"" + recipename + "\" ") ;

    // QSqlQuery query = db->exec(select);
    db->prepare(select);
    db->exec();

    if ( db->next() ) {
        return db->value(0).toInt(); 
    } else {
        return -1;
    }
}


/*
 *
 *
*/
int ScanXML::addRecipeToDatabase( Menu &menu ) {
    int recipeid=0;
    QValueList<Recipe> recipes = menu.getRecipe();
    MythProgressDialog *pd1 = new MythProgressDialog("Adding recipes to database",
		                                      recipes.size() );

    int cnt=1;
    // cout << "processing recipes in menu " << endl;
    for (QValueList<Recipe>::Iterator it=recipes.begin();
                                      it!=recipes.end();++it ) {
        Header head = (*it).getHeader();

        recipeid = addRecipeToDatabase( (*it) );

        if (pd1)
            pd1->setProgress( ++cnt );
    }

    if (pd1)
        delete pd1;

    return 0;
}



/*
 *
 *
*/
ScanXML::~ScanXML(void) {
	
    if (recipedb)
        delete recipedb;
}


/*
 *
 *
*/
void ScanXML::keyPressEvent(QKeyEvent *e ) {
    bool handled = false;
    QStringList actions;
    gContext->GetMainWindow()->TranslateKeyPress("Recipe", e, actions);

    // cout << "ScanSiteWindow::keyPressEvent() "
    // 	 << endl;

    for (unsigned int i = 0; i < actions.size() && !handled; i++) {
        QString action = actions[i];
        handled = true;

	cout <<"\t\t"<<action.data() << endl;

        if (action == "SELECT") {
	    enterPressed();
	}
        else
            handled = false;
    }

    if (!handled)
        MythThemedDialog::keyPressEvent(e);
    
    updateForeground();
}




/*
 *
 *
*/
void ScanXML::wireUpTheme() {

}
