/*
 * derived from mythgame
*/
#include <qapplication.h>
#include <qregexp.h>
#include <stdlib.h>
#include <iostream>
using namespace std;


#include <mythtv/mythcontext.h>
#include <mythtv/mythwidgets.h>
#include <mythtv/lcddevice.h>
#include <mythtv/xmlparse.h>

#include "Modulesettings.h"
#include "ViewRecipeWindow.h"
#include "recipe_filter.h"


#define RECIPE  3
#define CUISINE 2
#define COURSE  1


/*
 *
 *
*/
ViewRecipeWindow::ViewRecipeWindow(MythMainWindow *parent, MSqlQuery *ldb, 
	    QString window_name, QString theme_filename, 
            const char *name,
            const char *recipe_name)
           : MythThemedDialog(parent, window_name, theme_filename, name) 
{

    QString str;

    //  A few internal variable defaults
    if (ldb) {
        db = ldb;
    } else
        db = NULL;

    cout << "ViewRecipeWindow::ViewRecipeWindow() ... " 
         << recipe_name
         << endl;

    // setup volume for music
    //
    volume_control = NULL;
    volume_control = new VolumeControl(true);
    if (volume_control) {
        cout << "\tvolume control is available..."
             << endl;
    } else {
        cout << "\tvolume control is not available..."
             << endl;
    }
    showVolume(true);

    theme=NULL;

    wireUpTheme();

    if (!recipe_name) {
        QKeyEvent e(QEvent::KeyPress, Qt::Key_Escape, 27, Qt::KeyButtonMask);

        cout << "ViewRecipeWindow::ViewRecipeWindow()... no recipe given"
             << endl;
        MythThemedDialog::keyPressEvent(&e);
    } 
    //     displayRecipe( recipe_name );
}


/*
 *
 *
*/
ViewRecipeWindow::~ViewRecipeWindow(void) {

    cout << " ViewRecipeWindow::~ViewRecipeWindow(void)... closing..." << endl; 

}




/*
 *
 *
*/
void ViewRecipeWindow::displayRecipe( QString recipename) {
    QString thequery=0;
    QSqlQuery query;
    QString ingredients=0;
    QString str=0;
    QString ing_str=0;
    QStringList strlst;
    int recipeid=0;
    int authorid=0;

    cout << "\nViewRecipeWindow::displayRecipe()..." << endl; 

    if (db) {
	thequery = ("SELECT DISTINCT recipe_id, "        // 0
                                     "name, "            // 1
                                     "yield_qty, "       // 2
                                     "yield_units, "     // 3
                                     "source, "          // 4
                                     "author_user_id, "  // 5
                                     "comment, "         // 6
                                     "extra_comment, "   // 7
                                     "image_name, "      // 8
                                     "recipe_image "     // 9
                                     "FROM recipes WHERE "
                         "name=\"" + recipename + "\" ;" );
        // query = db->exec(thequery);
        db->prepare(thequery);
        db->exec();
            
        if (!db->next()) {
            cout << "\trecipe not found ..."
                 << endl;
	    return;
        }

        recipeid = (db->value(0).toInt());
        authorid = (db->value(5).toInt());

	if (recipeid<1)
	    return;

	if (serving_text) {
	    // serving_text->SetText("");
	    serving_text->SetText( db->value(2).toString()+" "+
                                   db->value(3).toString() );
	}

	if (source_text) {
	    // source_text->SetText("");
	    source_text->SetText( db->value(4).toString() );
	}

	if (comment_text)  {
	    comment_text->SetText( db->value(6).toString() +"\n\n"+ 
	         db->value(7).toString() );
        }

	if (recipe_image) {
            cout << "Debug: in if (recipe_image) ViewRecipeWindow.cpp ViewRecipeWindow::displayRecipe()" << endl;
	    QByteArray img_data = db->value(9).toByteArray();

            QFile file( "/tmp/recipe_image" );

            if ( file.open( IO_WriteOnly ) ) {
                    file.writeBlock(img_data);
                    file.close();
                    recipe_image->SetImage( "recipe_image" );
            } else {
                    QPixmap img(img_data);
                    recipe_image->SetImage( img );
                    cout << "load data from database to image..." << endl;
                    cout << "image width" << img.width() <<endl;
                    cout << "image height" << img.height() <<endl;
            }
	    recipe_image->LoadImage();
	    
            cout <<"ViewRecipeWindow::displayRecipe()... display image "<<endl; 
        } else 
            cout << "ViewRecipeWindow::displayRecipe()... no image " <<endl; 
	

	if (author_text) {
	    author_text->SetText("");
	    thequery.sprintf("SELECT DISTINCT user_name FROM users "
                             "WHERE user_id=%i;", query.value(5).toInt());
            // query = db->exec(thequery);
            db->prepare(thequery);
            db->exec();
            
            if (db->next()) {
	        author_text->SetText( db->value(0).toString() );
            } else 
	        author_text->SetText( "unKnown user" );
        }
	
	if (directions_list) {
            // directions_text->SetText("");
            directions_list->Reset();
            thequery.sprintf("SELECT step_num, step_text " 
			 "FROM instructions WHERE recipe_id=%i ;", recipeid );
	                
            // query = db->exec(thequery);
            db->prepare(thequery);
            db->exec();

            while (db->next()) {
              new UIListBtnTypeItem(directions_list,query.value(1).toString());
            }
        }
          
	thequery.sprintf("SELECT ingredient_id, recipe_id, ingredients.name, "
                         "display_order, ingredient_type, sub_recipe_id, qty, " 
                         "ingredients.unit_id, units.name, units.abbreviation "
                         "FROM ingredients, units "
                         "WHERE recipe_id=%i AND "
                         "ingredients.unit_id=units.unit_id;", recipeid );
        // query = db->exec(thequery);
        db->prepare(thequery);
        db->exec();
	

	if (ingredients_list) {
            QString ingredients;
	    ingredients_list->Reset();
            while (db->next()) {
                str = (db->value(2).toString());
                // ing_str = str.replace( QRegExp("\\d*||\[]"), "+");
                ing_str = str.replace(QRegExp("\\d*||transp.gif||\[]\[]"), "");
                switch (db->value(4).toInt()) {
                case 0 :
                case 1 :
                        ingredients.sprintf("%3.2f %s %s", 
					db->value(6).toString().toFloat(),
	                                db->value(9).toString().data(), 
                                        ing_str.data() );
			break;
                case 2 :
                        ingredients.sprintf("%s", ing_str.data() );
			break;
                case 3 :
                        ingredients.sprintf("%s", ing_str.data() );
			break;
                case 4 :
                        ingredients.sprintf("%s", ing_str.data() );
			break;
	        }
                new UIListBtnTypeItem(ingredients_list, ingredients.data());
            }
        }
    }
}


    

/*
 * handle keys/button sent to us...
*/
void ViewRecipeWindow::keyPressEvent(QKeyEvent *e) {
    bool handled = false;
    QStringList actions;
    gContext->GetMainWindow()->TranslateKeyPress("Recipe", e, actions);

    cout << "ViewRecipeWindow::keyPressEvent...\n" << endl;

    for (unsigned int i = 0; i < actions.size() && !handled; i++) {
        QString action = actions[i];
        handled = true;

	cout <<"\t\t"<<action.data() << endl;

        if (action == "SELECT") {
	    enterPressed();
	}
        else if (action == "MENU" || action == "INFO") {}
        else if (action == "UP")
	    cursorUp();
        else if (action == "DOWN")
	    cursorDown();
        else if (action == "LEFT")
            goLeft();
        else if (action == "RIGHT")
            goRight();
        else if (action == "PAGEUP") {}
        else if (action == "PAGEDOWN") {}
        else if (action == "TOGGLEFAV") {}
        else if (action == "MUTE") { 
	    if (muted) {
                if (volume_control)
	           volume_control->SetMute(FALSE);
	    } else {
                if (volume_control)
	           volume_control->SetMute(TRUE);
	    }
        }
        else if (action == "VOLUMEDOWN") {
            if (volume_control)
                volume_control->AdjustCurrentVolume(-1);
        }
        else if (action == "VOLUMEUP") {
            if (volume_control)
                volume_control->AdjustCurrentVolume(1);
        }
        else
            handled = false;
    }

    if (!handled)
        MythThemedDialog::keyPressEvent(e);
    
    updateForeground();
}



/*
 *
 *
*/
void ViewRecipeWindow::ingredients_list_enter_focus() {

    if (ingredients_list) {
	ingredients_list->SetActive(true);
	m_InColumn=4;
        if (info0_image) {
	    info0_image->SetImage( "active-box-0.png" );
	    info0_image->LoadImage();
        }
    }
    cout << "ViewRecipeWindow::ingredients_list_enter_focus...\n" << endl;
}
	


/*
 *
 *
*/
void ViewRecipeWindow::directions_list_enter_focus() {
    if (directions_list) {
	directions_list->SetActive(true);
       if (info3_image) {
           info3_image->SetImage( "background.png" );
           info3_image->LoadImage();
       }
    }
    cout << "ViewRecipeWindow::directions_list_enter_focus...\n" << endl;
    m_InColumn=3;
}
	


/*
 *
 *
*/
void ViewRecipeWindow::ingredients_list_loose_focus() {

    if (ingredients_list) {
	ingredients_list->SetActive(false);
        if (info0_image) {
    	    info0_image->SetImage( "inactive-box-0.png" );
    	    info0_image->LoadImage();
        }
    }
    cout << "ViewRecipeWindow::ingredients_list_loose_focus...\n" << endl;
    // updateForeground();
}
	


/*
 *
 *
*/
void ViewRecipeWindow::directions_list_loose_focus() {

    if (directions_list) {
	directions_list->SetActive(false);
        if (info0_image) {
    	    info0_image->SetImage( "" );
    	    info0_image->LoadImage();
        }
    }
    cout << "ViewRecipeWindow::directions_list_loose_focus...\n" << endl;
}
	


/*
 *
 *
*/
void ViewRecipeWindow::goRight() {
    QString str="";
    cout <<"ViewRecipeWindow::goRight()... column(" << ")" << endl;
}


/*
 *
 *
*/
void ViewRecipeWindow::goLeft() {
    cout <<"ViewRecipeWindow::goLeft()... column (" <<") " << endl;

}


/*
 *
 *
*/
void ViewRecipeWindow::numberPressAction(int ) {

}


/*
 *
 *
*/
void ViewRecipeWindow::cursorPgUp() {

}


/*
 *
 *
*/
void ViewRecipeWindow::cursorUp() {
    cout << "ViewRecipeWindow::cursorUp()... column (" << ")" << endl; 
}



/*
 *
 *
*/
void ViewRecipeWindow::cursorPgDown() {

}


/*
 *
 *
*/
void ViewRecipeWindow::cursorDown() {
    cout << "ViewRecipeWindow::cursorDown()... column (" << ")" << endl; 
}


/*
 *
 *
*/
void ViewRecipeWindow::enterPressed() {
    cout << "ViewRecipeWindow::enterPressed()... column (" << ")" << endl; 

}



/*
 *
 *
*/
void ViewRecipeWindow::showVolume(bool /* on_or_off */) {

}



/*
 *
 *
*/
/*
void ViewRecipeWindow::closeEvent(QCloseEvent *event) {
   // check if you need ambient music while you run other modules...
   //
   
   // if (need_ambient) {
   //     leave alone
   // } else {
   //     fmradio->();
   //     or 
   //     exec(mpg123 some.url)
   // }

    hide();
    event->accept();
}
*/




/*
 *
 *
*/
void ViewRecipeWindow::wireUpTheme() {
    QString str;

    // m_InColumn=3;

    // All the other GUI elements are **optional**

    author_text = getUITextType("author");
    if (!author_text) {
        cout << "\t\tunable to get author text ..." << endl;
    } else {
	author_text->SetJustification(Qt::AlignLeft|Qt::WordBreak);
	author_text->SetCutDown(true);
    }

    source_text = getUITextType("source");
    if (!source_text) {
        cout << "\t\tunable to get source text ..." << endl;
    } else {
	source_text->SetJustification(Qt::AlignLeft|Qt::WordBreak);
	source_text->SetCutDown(true);
    }

    serving_text = getUITextType("serving");
    if (!serving_text) {
        cout << "\t\tunable to get serving text ..." << endl;
    } else {
	serving_text->SetJustification(Qt::AlignLeft|Qt::WordBreak);
	serving_text->SetCutDown(true);
    }

    comment_text = getUITextType("comment");
    if (!comment_text) {
        cout << "\t\tunable to get comment text ..." << endl;
    } else {
	comment_text->SetJustification(Qt::AlignLeft|Qt::WordBreak);
	comment_text->SetCutDown(false);
    }



    current_recipe_text = getUITextType("current_recipe_text");
    if (!current_recipe_text) {
        cout << "\t\tunable to get current_recipe_text text ..." << endl;
    } else {
	current_recipe_text->SetJustification(Qt::AlignLeft|Qt::WordBreak);
	current_recipe_text->SetCutDown(false);
    }

    recipe_image = getUIImageType("recipeimage");
    if (!recipe_image) {
        cout << "\t\tunable to get recipe image ..." << endl;
    } else {
    }

    info0_image = getUIImageType("info0");
    if (!info0_image) {
        cout << "\t\tunable to get ingredients image ..." << endl;
    } else {
    }

    info1_image = getUIImageType("info1");
    if (!info1_image) {
        cout << "\t\tunable to get info image ..." << endl;
    } else {
    }

    info2_image = getUIImageType("info2");
    if (!info2_image) {
        cout << "\t\tunable to get directions image ..." << endl;
    } else {
    }

    info3_image = getUIImageType("filler");
    if (!info3_image) {
        cout << "\t\tunable to get filler image ..." << endl;
    } else {
    }

    info4_image = getUIImageType("info3");
    if (!info4_image) {
        cout << "\t\tunable to get filter image ..." << endl;
    } else {
    }

    directions_list = getUIListBtnType("directions");
    if (!directions_list) {
        cout << "\t\tunable to get directions text ..." << endl;
    } else {
	// directions_list->SetJustification(Qt::AlignLeft|Qt::WordBreak);
	directions_list->SetVisible(true);
	// directions_list->SetCutDown(true);
	directions_list->allowFocus(true);
	directions_list->calculateScreenArea();
    }

    
    ingredients_list = getUIListBtnType("ingredients");
    if (!ingredients_list) {
        cout << "\t\tunable to get ingredients list ..." << endl;
    } else {
        cout << "\t\tgot ingredients list ..." << endl;
	ingredients_list->SetVisible(true);
	ingredients_list->allowFocus( true );
	ingredients_list->SetActive( false );
	ingredients_list->calculateScreenArea();
    }

    // Focus enter events...
    if (ingredients_list)
         connect(ingredients_list, SIGNAL(takingFocus()),
                 this, SLOT( ingredients_list_enter_focus() ) );
    if (directions_list)
         connect(directions_list, SIGNAL(takingFocus()),
                 this, SLOT( directions_list_enter_focus() ) );

    // Focus loose events...
    if (ingredients_list)
         connect(ingredients_list, SIGNAL(loosingFocus()),
                 this, SLOT( ingredients_list_loose_focus() ) );
    if (directions_list)
         connect(directions_list, SIGNAL(loosingFocus()),
                 this, SLOT( directions_list_loose_focus() ) );
    
}
