/*
 * derived from mythgame
*/
#include <qapplication.h>
#include <qregexp.h>
#include <stdlib.h>
#include <iostream>
using namespace std;


#include <mythtv/mythcontext.h>
#include <mythtv/mythwidgets.h>
#include <mythtv/lcddevice.h>
#include <mythtv/xmlparse.h>

#include "Modulesettings.h"
#include "ModuleWindow.h"


#define RECIPE  3
#define CUISINE 2
#define COURSE  1


ModuleWindow::ModuleWindow(MythMainWindow *parent, QSqlDatabase *ldb, 
		    QString window_name, QString theme_filename, const char *name)
           : MythThemedDialog(parent, window_name, theme_filename, name)
{
    //  A few internal variable defaults
    if (ldb) {
        db = ldb;
    } else
        db = NULL;

    categorylist=NULL;
    recipelist=NULL;

    recipe_tree_root=NULL;
    recipe_tree_data=NULL;

    widget_focus=0;            // start focus on tree...
    
    // Through the magic of themes, our "GUI" already exists we just need to 
    // wire up it

    recipe_tree_root = new GenericTree("Choose a Course", 0, false);
    recipe_tree_data = recipe_tree_root->addNode(tr("All Recipes"), 0, false);

    theme=NULL;

    wireUpTheme();
    updateStatus();
    
    // Let's update the foreground just to be safe.
    recipe_tree_list->enter();
    updateForeground();
}


ModuleWindow::~ModuleWindow(void) {
}



/*
 * Return the type of node from the recipe tree
*/
int ModuleWindow::nodeType(GenericTree *node) {
    
    if ( ((node->getParent())->getParent()) ) {
        if ( ((node->getParent())->getParent())->getParent() ) {
	   return RECIPE;
	} else {
	   return CUISINE;
        }
    } else {
        return COURSE;
    }
}



/*
 * handle keys/button preeses sent to us...
*/
void ModuleWindow::keyPressEvent(QKeyEvent *e) {
    bool handled = false;
    QStringList actions;
    gContext->GetMainWindow()->TranslateKeyPress("Recipe", e, actions);

    cout << "ModuleWindow::keyPressEvent...\n" << endl;

    for (unsigned int i = 0; i < actions.size() && !handled; i++) {
        QString action = actions[i];
        handled = true;

	cout <<"\t\t"<<action.data() << endl;

        if (action == "SELECT") {
            recipe_tree_list->enter();
	}
        else if (action == "MENU" || action == "INFO") 
            edit();
        else if (action == "UP")
	    cursorUp();
        else if (action == "DOWN")
	    cursorDown();
        else if (action == "LEFT")
            goLeft();
        else if (action == "RIGHT")
            goRight();
        else if (action == "PAGEUP")
            recipe_tree_list->pageUp();
        else if (action == "PAGEDOWN")
            recipe_tree_list->pageDown();
        else if (action == "TOGGLEFAV")
            recipe_tree_list->select();
        else
            handled = false;
    }

    if (!handled)
        MythThemedDialog::keyPressEvent(e);
}



void ModuleWindow::categoryListChanged(int) {

}



void ModuleWindow::recipeListChanged(int) {

}



void ModuleWindow::ingredientsListChanged(int item) {
    cout << "ModuleWindow::ingredientListChanged()...\n";

    if (ingredientslist) {
        cout << "\tgot "<< (ingredientslist->GetItemAt(item))->text().data() << "\n";
	ingredient_text->SetText( (ingredientslist->GetItemAt(item))->text() );
    }
}



void ModuleWindow::updateList( QString name, QRect listRect) {
    cout << "ModuleWindow::updateList()... start updating " << name <<" ..."<<endl;
    QPixmap pix(listRect.size());
    pix.fill(this, listRect.topLeft());
    QPainter p(&pix);
    
    LayerSet* container = theme->GetSet(name);
    if (container) {
        container->Draw(&p, 0, 0);
        container->Draw(&p, 1, 0);
        container->Draw(&p, 2, 0);
        container->Draw(&p, 3, 0);
        container->Draw(&p, 4, 0);
        container->Draw(&p, 5, 0);
        container->Draw(&p, 6, 0);
        container->Draw(&p, 7, 0);
        container->Draw(&p, 8, 0);
    } else 
       cout << "\t\tcould not get " << name <<" ..."<<endl;;
    p.end();
    
    bitBlt(this,listRect.left(), listRect.top(),
           &pix, 0, 0, -1, -1, Qt::CopyROP);
}



void ModuleWindow::updateRecipelistTreeTitle( GenericTree *node ) {
    QString str="";
                        switch (nodeType(node)) {
                        case COURSE:
				str=("Select a Course... \n");
		                break;
                        case CUISINE:
				str=("Course... "+
					((node->getParent())->getString())+
					"\n    Select a Cuisine... \n");
		                break;
                        case RECIPE:
                                // m_InColumn=6;
				str=("Cuisine... "+
					((node->getParent())->getString())+
					" "+
				((node->getParent()->getParent())->getString())+
					" ...\n    Select a Recipe... \n");
		                break;
                        }
                    if (recipetreelisttitle_text)
                        recipetreelisttitle_text->SetText(str);
}



void ModuleWindow::goLeft() {
    cout <<"ModuleWindow::goLeft()..." << endl;

        switch (m_InColumn) {
            case 3 :
                    if (recipe_tree_list) {
                        recipe_tree_list->popUp();
                        GenericTree *node = recipe_tree_list->getCurrentNode();
                        switch (nodeType(node)) {
                            case COURSE:
		                break;
                            case CUISINE:
		                break;
                            case RECIPE:
                                // m_InColumn=6;
		                break;
                        }
                        recipe_tree_list->enter();
                    }
                    updateRecipelistTreeTitle( recipe_tree_list->getCurrentNode());
                    break;
            case 4 :
                    m_InColumn=3;
                    break;
            case 5 :
                    m_InColumn=4;
                    break;
        }
	
    setColumnActive( m_InColumn );
    updateList("ingredientsarea", ingredientslistRect);
}




void ModuleWindow::goRight() {
    QString str="";
    cout <<"ModuleWindow::goRight()..." << endl;

        switch (m_InColumn) {
            case 3 :
                    if (recipe_tree_list) {
                        GenericTree *node = recipe_tree_list->getCurrentNode();
                        switch (nodeType(node)) {
                        case CUISINE:
				if (node->childCount()<1) {
		                    addRecipeToList(node);
				}
                                recipe_tree_list->pushDown();
		                break;
                        case RECIPE:
				if (ingredientslist) {
                                    m_InColumn=4;
				}
		                break;
                        case COURSE:
		                if ( (node->getString())=="All Recipes" ) {
		                    if ( (node->childCount()<1) ) {
                                        node->addNode("All Recipes", 0, false);
		                    }
		                }
                                recipe_tree_list->pushDown();
		                break;
                        }
			updateRecipelistTreeTitle( 
					recipe_tree_list->getCurrentNode());
                        recipe_tree_list->enter();
                    }
                    break;
            case 4 :
                    m_InColumn=5;
                    break;
            case 5 :
                   if (recipe_tree_list) 
                       m_InColumn=3;
                    break;
        }
    setColumnActive( m_InColumn );
    updateList("ingredientsarea", ingredientslistRect);
}






void ModuleWindow::addRecipeToList(GenericTree *node) {
    QString thequery;
    QSqlQuery query;
    QString parent;
    QString child;

    cout << "ModuleWindow::addRecipeToList()..\n";

    if (nodeType(node)==CUISINE) {
        if (node->childCount()<1) {
            if (db) {
		parent=(node->getParent())->getString();
		child=node->getString();
		if ( (child=="All Recipes") ) {
		    thequery.sprintf("SELECT name FROM recipes ORDER BY name");
                    query = db->exec(thequery);
                    while (query.next()) {
                         node->addNode(query.value(0).toString(), 0, false);
		    }
		} else {
		    // get parent course
		    thequery.sprintf("SELECT category_id FROM categories WHERE parent=0 AND name=\"%s\"", parent.data());
                    query = db->exec(thequery);
		    query.next();

		    thequery.sprintf("SELECT DISTINCT category_id FROM categories WHERE parent=%i AND name=\"%s\"", query.value(0).toInt(), child.data());
                    query = db->exec(thequery);
		    query.next();

                    thequery.sprintf("SELECT recipes.name FROM category_entries, recipes WHERE category_entries.category_id=%i AND recipes.recipe_id=category_entries.recipe_id ORDER BY recipes.name ;", query.value(0).toInt());
                    query = db->exec(thequery);
                
                    while (query.next()) {
                         node->addNode(query.value(0).toString(), 0, false);
		    }
		}
            }
	}
    } 
}


void ModuleWindow::edit() {
    cout <<"ModuleWindow::edit..." << endl;
}


void ModuleWindow::previous() {
}

void ModuleWindow::next() {
}


void ModuleWindow::seeknext() {
}

void ModuleWindow::seekprev() {
}


void ModuleWindow::stop() {
}

void ModuleWindow::record() {
}

void ModuleWindow::visEnable() {
}


void ModuleWindow::updateStatus() {
}




void ModuleWindow::handleTreeListSelection(int , IntVector*) {
    GenericTree *node = recipe_tree_list->getCurrentNode();
    cout << "ModuleWindow::handleTreeListSelection().." << endl; 
    
    switch (nodeType(node)) {
        case CUISINE:
		break;
        case RECIPE:
		break;
        case COURSE:
		break;
    }

}




void ModuleWindow::displayDirections(int /* topdirections */) {
    QString directionstext="";
    QStringList::Iterator it;
    int cnt=-1;

    cout << "ModuleWindow::displayDirections()..." << endl; 

    if (!directions_text)
        return;

    if (directions_top<0)
        directions_top=0;

    for (it=cur_directions.begin();it!=cur_directions.end();++it){
	cnt++;
	if (cnt>directions_top) {
            directionstext.sprintf("%s%i. %s\n",directionstext.data(),cnt,(*it).data());
	}
    }

    if (directionstext.isEmpty()) {
        directionstext.sprintf("%s%i. %s\n",directionstext.data(),cnt,(*(--it)).data());
	directions_top=cnt-1;
    }

    directions_text->SetText(directionstext);
    // updateList("ingredientsarea", ingredientslistRect);
}



void ModuleWindow::displayRecipe( QString recipename) {
    QString thequery;
    QSqlQuery query;
    QString ingredients;
    QString str;
    QStringList strlst;

    int recipeid=0;
    int authorid=0;

    cout << "ModuleWindow::displayRecipe()..." << endl; 

    if (db) {
	thequery.sprintf("SELECT DISTINCT recipe_id, name, \
			         yield_qty, yield_units, source, author_user_id \
			 FROM recipes WHERE name=\"%s\";", recipename.data());
        query = db->exec(thequery);
        query.next();
        recipeid = (query.value(0).toInt());
        authorid = (query.value(5).toInt());

	if (recipeid<1)
	    return;

        str = (query.value(4).toString());

	if (source_text)
	    source_text->SetText( str.section('|',0,0).data() );
	if (serving_text)
	    serving_text->SetText( query.value(3).toString().data() );
	if (recipe_image) {
	    QString img;
	    img = "p./"+str.section('|',1,1);
	    recipe_image->SetImage( img );
	    recipe_image->LoadImage();
        }
	

	if (author_text) {
	    thequery.sprintf("SELECT DISTINCT user_id, user_name \
			 FROM users WHERE user_id=%i;", authorid);
            query = db->exec(thequery);
            query.next();
	    author_text->SetText( query.value(1).toString().data() );
        }

	if (directions_text) {
	    thequery.sprintf("SELECT step_num, step_text \
			 FROM instructions \
			 WHERE recipe_id=%i ;", recipeid );
            query = db->exec(thequery);

	    query.next();
	    if (comment_text)
	        comment_text->SetText( query.value(1).toString().data() );

	    query.next();
	    directions_cnt=1;
	    directions_top=0;
	    cur_directions = QStringList::split("\n", query.value(1).toString().data());
	    displayDirections(directions_top);
        }
	thequery.sprintf("SELECT ingredient_id, recipe_id, ingredients.name, \
			         display_order, ingredient_type, sub_recipe_id, \
				 qty, ingredients.unit_id, units.name, units.abbreviation \
			 FROM ingredients, units \
		     WHERE recipe_id=%i AND ingredients.unit_id=units.unit_id;", recipeid );
        query = db->exec(thequery);

	if (ingredientslist)
	    ingredientslist->Reset();

        while (query.next()) {
            switch (query.value(4).toInt()) {
                case 1 :
                        ingredients.sprintf("%2.0f %s %s", query.value(6).toDouble(),
	                                    query.value(8).toString().data(),
	                                    query.value(2).toString().data());
			break;
                case 2 :
                        ingredients.sprintf("2-%s",query.value(2).toString().data());
			break;
                case 3 :
                        ingredients.sprintf("3-%s",query.value(2).toString().data());
			break;
                case 4 :
                        ingredients.sprintf("4-%s",query.value(2).toString().data());
			break;
                case 0 :
                        ingredients.sprintf("0-%s",query.value(2).toString().data());
			break;

	    }
	    
	    if (ingredientslist) {
                UIListBtnTypeItem *item = 
			new UIListBtnTypeItem(ingredientslist, ingredients.data());
	        // ingredientslist->AddItemAt(ingredients.data());
            }
        }
    }
    updateList("ingredientsarea", ingredientslistRect);
}



void ModuleWindow::handleTreeListEntry(int, IntVector*) {
    GenericTree *node = recipe_tree_list->getCurrentNode();
    cout << "ModuleWindow::handleTreeListEntry().." << endl;

    switch (nodeType(node)) {
        case CUISINE:
		break;
        case RECIPE:
		displayRecipe(node->getString());
		break;
        case COURSE:
		break;
    }
}



void ModuleWindow::fillRecipeTree( QString /* filter */) {
    cerr << "ModuleWindow::fillRecipeTree: ..." << endl;
    recipe_tree_list->showWholeTree(true);
    recipe_tree_list->colorSelectables(true);
	
    if (db) {
        QString thequery="SELECT DISTINCT category_id, name FROM categories WHERE parent=0 ORDER BY name;";
        QSqlQuery query = db->exec(thequery);
	    
        while (query.next()) {
            recipe_tree_data=recipe_tree_root->addNode(query.value(1).toString(), 0, false);
            QString crsquery;
            crsquery.sprintf("SELECT DISTINCT category_id, name FROM categories WHERE parent=%i ORDER BY name;", query.value(0).toInt());
            QSqlQuery cquery = db->exec(crsquery);

            while (cquery.next()) {
                recipe_tree_data->addNode(cquery.value(1).toString(), 0, false);
            }
        }
    }

    recipe_tree_list->assignTreeData(recipe_tree_root);

    connect(recipe_tree_list, SIGNAL(nodeSelected(int, IntVector*)), 
                this, SLOT(handleTreeListSelection(int, IntVector*)));
    connect(recipe_tree_list, SIGNAL(nodeEntered(int, IntVector*)), 
                this, SLOT(handleTreeListEntry(int, IntVector*)));
}



	
void ModuleWindow::wireUpTheme() {
    QString str;

    m_InColumn=3;

    // All the other GUI elements are **optional**

    recipe_tree_list = getUIManagedTreeListType("recipetreelist");
    if (!recipe_tree_list) {
    } else {
	fillRecipeTree("");
    }

    recipetreelisttitle_text = getUITextType("recipetreelisttitle");
    if (!recipetreelisttitle_text) {
    } else {
	recipetreelisttitle_text->SetJustification(Qt::AlignLeft|Qt::WordBreak);
	recipetreelisttitle_text->SetCutDown(false);
    }

    author_text = getUITextType("author");
    if (!author_text) {
    } else {
	author_text->SetJustification(Qt::AlignLeft|Qt::WordBreak);
	author_text->SetCutDown(true);
    }

    source_text = getUITextType("source");
    if (!source_text) {
    } else {
	source_text->SetJustification(Qt::AlignLeft|Qt::WordBreak);
	source_text->SetCutDown(true);
    }

    serving_text = getUITextType("serving");
    if (!serving_text) {
    } else {
	serving_text->SetJustification(Qt::AlignLeft|Qt::WordBreak);
	serving_text->SetCutDown(true);
    }

    comment_text = getUITextType("comment");
    if (!comment_text) {
    } else {
	comment_text->SetJustification(Qt::AlignLeft|Qt::WordBreak);
	comment_text->SetCutDown(false);
    }

    recipe_image = getUIImageType("recipeimage");
    if (!recipe_image) {
    } else {
    }

    info0_image = getUIImageType("info0");
    if (!recipe_image) {
    } else {
    }

    info1_image = getUIImageType("info1");
    if (!recipe_image) {
    } else {
    }

    info2_image = getUIImageType("info2");
    if (!recipe_image) {
    } else {
    }

    directions_text = getUITextType("directions");
    if (!directions_text) {
    } else {
	directions_text->SetJustification(Qt::AlignLeft|Qt::WordBreak);
	directions_text->SetCutDown(true);
    }

    ingredient_text = getUITextType("ingredient");
    if (!ingredient_text) {
    } else {
	ingredient_text->SetJustification(Qt::AlignLeft|Qt::WordBreak);
	ingredient_text->SetCutDown(true);
    }

    theme = loadTheme("Recipe");
    
    // recipe_tree_list = getUIManagedTreeListType("recipetreelist");

    if (!theme) {
    } else {
        LayerSet *container = theme->GetSet("recipetreelistarea");
        if (!container) {
        } else {
	    recipetreelistRect=container->GetAreaRect();
            // recipetreelist = (UIListBtnType*)container->GetType("ingredients");
            // if (!recipetreelist) {
            // } else {
            // }
        }
        container = theme->GetSet("ingredientsarea");
        if (!container) {
        } else {
	    ingredientslistRect=container->GetAreaRect();
            ingredientslist = (UIListBtnType*)container->GetType("ingredients");
            if (!ingredientslist) {
            } else {
            }
        }
        container = theme->GetSet("recipelistarea");
        if (!container) {
        } else {
	    if (recipe_tree_list) {
	    } else {
	        // if there is no managed recipe tree list then 
	        // get a recipe_list and category_list
	        recipelistRect=container->GetAreaRect();
                recipelist = (UIListBtnType*)container->GetType("recipelist");
                if (!recipelist) {
                } else {
                }
	    }
        }
        container = theme->GetSet("categorylistarea");
        if (!container) {
        } else {
	    if (recipe_tree_list) {
	    } else {
	        // if there is no managed recipe tree list then 
	        // get a recipe_list and category_list
	        categorylistRect=container->GetAreaRect();
                categorylist = (UIListBtnType*)container->GetType("categorylist");
                if (!categorylist) {
                } else {
                }
	    }
        }
    }

    if (recipetreelisttitle_text)
        recipetreelisttitle_text->SetText("Select a Course...");

     connect(categorylist, SIGNAL(itemSelected(int)),
             SLOT(categoryListChanged(int)));
     connect(recipelist, SIGNAL(itemSelected(int)),
             SLOT(recipeListChanged(int)));
     connect(ingredientslist, SIGNAL(itemSelected(int)),
             SLOT(ingredientsListChanged(int)));
    
    setColumnActive(m_InColumn);
}







XMLParse *ModuleWindow::loadTheme(QString theme)
{
    XMLParse *them = new XMLParse();
    them->SetWMult(wmult);
    them->SetHMult(hmult);

    QDomElement xmldata;
    QString theme_;

    theme_.sprintf("%s-", theme.data());

    them->LoadTheme(xmldata, theme.data(), theme_.data());

    for (QDomNode child = xmldata.firstChild(); !child.isNull();
         child = child.nextSibling()) {

        QDomElement e = child.toElement();
        if (!e.isNull()) {

            if (e.tagName() == "font") {
                them->parseFont(e);
            } else if (e.tagName() == "container") {
                QRect area;
                QString name;
                int context;

                them->parseContainer(e, name, context, area);

                if (name.lower() == "ingredients") {
                    ingredientslistRect = area;
		} else if (name.lower() == "recipes")
                    recipelistRect = area;
            } else {
                std::cerr << "Unknown element: " << e.tagName()
                          << std::endl;
            }
        }
    }

    return them;
}






/*
*/
void ModuleWindow::numberPressAction(int numberkey ) {
    if (m_InColumn==2) {
        // number keys move to percentage in list of recipes
        if (recipelist) {
            int currentpos = recipelist->GetItemPos(recipelist->GetItemCurrent());
	    int numrecipesinlist = recipelist->GetCount();
	    int movetopos  = (int)(numrecipesinlist * (numberkey*.1));
	    if (movetopos<currentpos) {
                for (int i=currentpos; i>movetopos; i--)
		    recipelist->MoveUp();
	    } else {
                for (int i=currentpos; i<movetopos; i++)
		    recipelist->MoveDown();
	    }
        }
    } else {
    }
    // updateList("ingredientsarea", ingredientslistRect);
}



/*
*/
void ModuleWindow::cursorPgUp()
{
    if ( (categorylist) && (m_InColumn == 0) ) {
        categorylist->MoveUp();
    } else if ( (recipelist) && (m_InColumn == 2) ){
            recipelist->MoveUp();
    } 
    
    // update();
}

void ModuleWindow::cursorUp()
{
    cout << "ModuleWindow::cursorUp()...\n"; 

    if (1) {
        switch (m_InColumn) {
	    case 0:
		   break;
	    case 1:
		   break;
	    case 2:
		   break;
	    case 3:
                   if (recipe_tree_list)
		       recipe_tree_list->moveUp();
		   break;
	    case 4:
                   if (ingredientslist)
		       ingredientslist->MoveUp();
                       updateList("ingredientsarea", ingredientslistRect);
		   break;
	    case 5:
		   displayDirections(--directions_top);
		   break;
	    default:
		   break;
	}
    }
    
    // update();
    updateList("ingredientsarea", ingredientslistRect);
}

void ModuleWindow::cursorPgDown()
{
    if ( (categorylist) && (m_InColumn == 0) ) {
        categorylist->MoveDown();
    }
    else if ( (recipelist) && (m_InColumn == 2) ){
            recipelist->MoveDown();
	    displayRecipe((recipelist->GetItemCurrent())->text().data());
    } 

    // update();
}

void ModuleWindow::cursorDown()
{
    cout << "ModuleWindow::cursorDown()...\n"; 
        switch (m_InColumn) {
	    case 0:
		   break;
	    case 1:
		   break;
	    case 2:
		   break;
	    case 3:
                   if (recipe_tree_list)
		       recipe_tree_list->moveDown();
		   break;
	    case 4:
                   if (ingredientslist)
		       ingredientslist->MoveDown();
                       updateList("ingredientsarea", ingredientslistRect);
		   break;
	    case 5:
		   displayDirections(++directions_top);
		   break;
	    default:
		   break;
	}
    // update();
    updateList("ingredientsarea", ingredientslistRect);
}



void ModuleWindow::cursorLeft()
{
    int prevcol=m_InColumn;
    
    if (m_InColumn==0) {
        m_InColumn=2;
    } else {
        m_InColumn--;
    }

    // set recipelist as default view if no theme for category list or alphabet list
    if ( (m_InColumn==0) && (!(categorylist)) ) 
        m_InColumn=2;

    if ( (prevcol==0) && (m_InColumn==2) ) {
       // we are moving from categories to recipes so fill recipes from category
       buildRecipeList( false, 0);
    } else {
        if ( (prevcol==1) && (m_InColumn==2) ) {
           // we are moving from categories to alphabet list so fill recipes by alphabet
           buildRecipeList( true, 1);
        } else {
	    // goign from alpha to category/back
	}
    }
    /*
    if ( (m_InColumn==2) && (!(recipelist)) ) 
        m_InColumn=1;
    */
    setColumnActive(m_InColumn);
    updateList("ingredientsarea", ingredientslistRect);
}


void ModuleWindow::cursorRight()
{
    int prevcol=m_InColumn;

    if (m_InColumn==2) {
        m_InColumn=0;
    } else {
        m_InColumn++;
    }

    // set recipelist as default view if no theme for category list or alphabet list
    if ( (m_InColumn==0) && (!(categorylist)) ) 
        m_InColumn=2;

    if ( (prevcol==0) && (m_InColumn==2) ) {
       // we are moving from categories to recipes so fill recipes from category
       buildRecipeList( false, 0);
    } else {
        if ( (prevcol==1) && (m_InColumn==2) ) {
           // we are moving from categories to alphabet list so fill recipes by alphabet
           buildRecipeList( true, 1);
        } else {
	    // goign from alpha to category/back
	}
    }

    setColumnActive(m_InColumn);
    updateList("ingredientsarea", ingredientslistRect);
}




/*
*/
void ModuleWindow::setColumnActive(int m_InColumn ) {

    switch (m_InColumn) {
        case 0:
            if (recipelist) recipelist->SetActive(false);
            if (categorylist) categorylist->SetActive(true);
		break;
        case 1: 
            if (recipelist) recipelist->SetActive(false);
            if (categorylist) categorylist->SetActive(false);
		break;
        case 2: 
            if (recipelist) recipelist->SetActive(true);
            if (categorylist) categorylist->SetActive(false);
		break;
        case 3: 
            if (recipelist) recipelist->SetActive(false);
            if (categorylist) categorylist->SetActive(false);
            if (ingredientslist) ingredientslist->SetActive(false);
	    break;
        case 4: 
            if (recipelist) recipelist->SetActive(false);
            if (categorylist) categorylist->SetActive(false);
            if (ingredientslist) {
		ingredientslist->SetActive(true);
		ingredientslist->MoveUp();
	    }
            break;
        case 5: 
            if (recipelist) recipelist->SetActive(false);
            if (categorylist) categorylist->SetActive(false);
            if (ingredientslist) ingredientslist->SetActive(false);
            break;
        case 6: 
            if (recipelist) recipelist->SetActive(false);
            if (categorylist) categorylist->SetActive(false);
            if (ingredientslist) ingredientslist->SetActive(true);
            break;
        default: 
            if (recipelist) recipelist->SetActive(false);
            if (categorylist) categorylist->SetActive(false);
    }

    update();
    // updateList("ingredientsarea", ingredientslistRect);
}



void ModuleWindow::volumeup() {
    changeVolume(TRUE);
}



void ModuleWindow::volumedown() {
    changeVolume(FALSE);
}



void ModuleWindow::changeVolume(bool up_or_down) {
    if (volume_control) {
        if (up_or_down)
            volume_control->AdjustCurrentVolume(2);
        else
            volume_control->AdjustCurrentVolume(-2);
        showVolume(true);
    }
}



void ModuleWindow::toggleMute() {
    if (volume_control) {
        volume_control->ToggleMute();
        showVolume(true);
    }
}



void ModuleWindow::showVolume(bool /* on_or_off */) {
}



void ModuleWindow::updateStatus( QString ) {
}



void ModuleWindow::closeEvent(QCloseEvent *event) {
   // check if you need ambient music while you run other modules...
   //
   
   // if (need_ambient) {
   //     leave alone
   // } else {
   //     fmradio->();
   //     or 
   //     exec(mpg123 some.url)
   // }

    hide();
    event->accept();
}


void ModuleWindow::showEvent(QShowEvent *event) {
    QWidget::showEvent(event);
}


void ModuleWindow::customEvent(QCustomEvent *event) {
    QWidget::customEvent(event);
}




void ModuleWindow::buildRecipeList( bool all, int cur_list ) {
QString category="";
QSqlQuery query;
int categoryid=-1;


    cout << "RecipeWindow::buildRecipeList()... start...\n";

    if (recipelist) {
	recipelist->Reset();

        //build a list of all the recipe categories there are
        if (db) {
            QString thequery;
	    if (all) {
                thequery = "SELECT DISTINCT name FROM recipes ORDER BY name;";
	    } else {
                if (cur_list==0) {
		    // category column was chosen so fill recipe list with current category...
		    // UIListBtnType::UIListBtnTypeItem* cur_item;

                    category = (categorylist->GetItemCurrent())->text();
                    // category = cur_item->text;
		} else {
		    category = "All Categories ";
		}
                
                thequery.sprintf(
		    "SELECT category_id FROM categories WHERE name=\"%s\";",
			 category.data());
                query = db->exec(thequery);
                query.next();
                categoryid = query.value(0).toInt();
		
                if (categoryid==(0)) {
                    thequery = "SELECT DISTINCT name FROM recipes ORDER BY name;";
		} else
                    thequery.sprintf("SELECT recipes.name \
		    		        FROM category_entries, recipes \
		    	        WHERE category_entries.category_id=%i \
		   	    AND recipes.recipe_id=category_entries.recipe_id \
			    ORDER BY recipes.name ;", categoryid);
	    }
            query = db->exec(thequery);
            while ( query.next() ) {
                UIListBtnTypeItem *item =
                         new UIListBtnTypeItem(recipelist, query.value(0).toString());
            }
        } else
            cout << "\t\trecipelist not defined...\n";
    } else
        cout << "\t\trecipelist not defined...\n";

}


void ModuleWindow::buildCategoryList() {

    if (categorylist) {
        UIListBtnTypeItem *item =
		new UIListBtnTypeItem(categorylist, "All Categories ");
        cout << "RecipeWindow::buildCategoryList()... start...\n";
        //build a list of all the recipe categories there are
        if (db) {
           QString thequery=
	   "SELECT DISTINCT name FROM categories WHERE parent=0 ORDER BY name;";
            QSqlQuery query = db->exec(thequery);
                while (query.next()) {
                    // categorylist->AddItem(query.value(0).toString());
                    UIListBtnTypeItem *item =
		            new UIListBtnTypeItem(categorylist, query.value(0).toString());
                }
        }
    } else
        cout << "\t\tcategory list not defined..." << endl;
}


