#ifndef MENUDEFINE_H
#define MENUDEFINE_H

#include <stdlib.h>
#include <iostream>
using namespace std;

#include <qstring.h>
#include <qstringlist.h>
#include <qvaluelist.h>

#define Quantity float

class Range {

    public:
        Range()
            : a_from(0),
              a_to(0)   {}
        int from() {return a_from;}
        int to() {return a_to;}
        bool zero() {return (a_to==a_from);}

        void setValue(int &from, int &to) {a_from=from;a_to=to;}
        void setFrom(int &from) {a_from=from;}
	void setTo(int &to) {a_to=to;}

    private:
        int a_from, a_to;
};


class Amount {

    public:

        Amount()
	    : a_qty(0),
	      a_unit(""),
	      a_size("") {}

        Amount(float qty, QString unit, QString size)
	    : a_qty(qty),
	      a_unit(unit),
	      a_size(size) {}
        Amount(float from, float to, QString unit, QString size)
	    : a_qty(0.0),
	      a_unit(unit),
	      a_size(size) {}

        float &qty() {return a_qty;}
        Range  &range() {return a_range;}
	QString &unit() {return a_unit;}
	QString &size() {return a_size;}
        QString value();

        void setQuantity(float qty) {a_qty=qty;}
        void setSize(QString size) {a_size=size;}
        void setUnit(QString unit) {a_unit=unit;}
        void setRange(Range r) {a_range=r;}

        void display(QString str="");

    private:
        float a_qty;
        Range a_range;
	QString a_unit;
	QString a_size;
};


class Header {
    public:
        Header()
           : title("empty"),
             sub_title(""),
             version("0.0"),
             source("xml") {}

	void setTitle( QString str) {title=str;}
	void setSubTitle( QString str) {sub_title=str;}
	void setVersion( QString str) {version=str;}
	void setSource( QString str) {source=str;}
	void addCategory( QString str) {category.append(str);}
	void setYield( Amount amt) {yield=amt;}

        QString &getTitle() {return title;}
        QString &getAuthor() {return author;}
        QString &getSubTitle() {return sub_title;}
        QString &getVersion() {return version;}
        QString &getSource() {return source;}
        QValueList<QString> &getCategory() {return category;}
        QValueList<QString> &getPrepTime() {return preptime;}
        Amount &getYield() {return yield;}

        void clearCategory() {category.clear();}

        void display(QString str="") ;

    private:
        QString     title;
        QString     sub_title;
        QString     version;
        QString     author;
        QString     source;
        QValueList<QString> category;
        Amount      yield;
        QValueList<QString> preptime;
};


class Nutrition {

    public:
        Nutrition()
             : nutrition("") {}
        QString& data() {return nutrition;}
        void setData( QString n ) {nutrition = n;}

    private:
        QString nutrition;
};

class Yield {
    public:
        Yield()
            : unit("")   {};
        
    private:
        QValueList<Quantity> qty;
        Range                range;
        QString              unit;
};


class PrepTime {
    public:
        PrepTime();
        ~PrepTime();
        
        QValueList<QString> preparation;
};


class Description {
    public:
        Description() { }
        QString data( int ) {return "";}
	void setData( QString line ) {}
    private:
        // QStringList lines;
};



class IngItem {

    public:
        IngItem();
        ~IngItem();
        
	void   setType(QString t) {type=t;}
	void   setItem(QString i) {item=i;}
	void   setPrep(QString p) {prep=p;}
	void   setModifier(QString m) {modifier=m;}

	void   addAmount(Amount a) {amt.append(a);}
	void   clearAmount() {amt.clear();}
        QValueList<Amount> &getAmount() {return amt;}

	void   addNote(QString n) {notes.append(n);}
	void   clearNotes() {notes.clear();}
        QValueList<QString> &getNotes() {return notes;}

	void   addProdCode(QString p) {prodcode.append(p);}
	void   clearProdCode() {prodcode.clear();}
        QValueList<QString> &getProdCode() {return prodcode;}

        void   display(QString str="") ;

        QString &getType() {return type;}
        QString &getItem() {return item;}
        QString &getPrep() {return prep;}
        QString &getModifier() {return modifier;}

    private:
        QString             type;  // ing-div (0 if none)
	QValueList<Amount>  amt;
        QString             item;
        QString             prep;
        QString             modifier;
        QValueList<QString> notes;
        QValueList<QString> prodcode;
        // QValueList<IngItem> alt_ing;
};


class Ingredient {
    // Created based whenver there is a new <ingredients>
    // tag.  Contains optional note(s) and a list of Ingredient
    // items (each with a <ing> tag
    public:
        Ingredient()
               : title(""),
                 description("") {};
        ~Ingredient();

        void setTitle(QString t) {title=t;}
        QString &getTitle() {return title;}

        void setDescription( QString d) {description=d;}
        QString &getDescription() {return description;}

	void addIngItem(IngItem item) {ing_items.append( item );}
	QValueList<IngItem> &getIngItems() {return ing_items;}

        void addNote( QString note) {notes.append( note );}
	QValueList<QString> &getNotes() {return notes;}

        void display(QString str="") ;

    private:
        QString             title;
        QString             description;
        QValueList<QString> notes;
        QValueList<IngItem> ing_items;
};


class DietRequirements {

    public:
        DietRequirements() {}
        QString data() {return diet;}
        void setData( QString d) {diet = d;}

    private:
        QString diet;
};

class Recipe {

    public:
        Recipe();
        ~Recipe();

	unsigned int  numberOfIngredients() const {return ingredients.size();}

        void addIngredient( Ingredient ing ) { ingredients.append( ing );}

	Header& getHeader() {return header;}
	void setHeader( Header h ) { header=h; }

        QValueList<Description> &getDescription() {return descriptions;}
        void addDescription( Description d ) {descriptions.append( d );}
        void clearDescription() { descriptions.clear();}

        QValueList<QString> &getDirection() {return directions;}
	void addDirection( QString d ) { directions.append( d );}
        void clearDirection() { directions.clear();}

	QValueList<Ingredient> &getIngredient() {return ingredients;}

        void display(QString str="");

        QByteArray &getImageData() {return image;}
        void setImageData(QByteArray id) {image=id;}

    private:
        Header                       header;
        QString                      equiptment;
        QValueList<Ingredient>       ingredients;
        QValueList<QString>          directions;
	QValueList<Nutrition>        nutrition;
	QValueList<DietRequirements> diet;
        QValueList<Description>      descriptions;
	QByteArray                   image;
};


class ShoppingList {

    public:
        ShoppingList();
        ~ShoppingList();

        void addRecipeIngredients( Recipe );

        Header& getHeader() {return header; }
	QValueList<Description> &getDescription( );
        void resetShoppingList(); 
        void addDescription( Description d ) {descriptions.append( d );}
	
    private:
        Header                  header;
        QValueList<Description> descriptions;
};

class Menu {

    public:
        Menu();
        ~Menu();


	void addRecipe( Recipe& recipe) {recipes.append(recipe); }
	Recipe& getRecipe( unsigned int r ) {return recipes[r];}
	QValueList<Recipe>& getRecipe( ) {return recipes;}
	void clearRecipes() {recipes.clear();}

	unsigned int  numberOfRecipes() const {return recipes.size();}

	QValueList<ShoppingList> &getShoppingList();

	Header&  getHeader() {return header;}
	void setHeader( Header h ) { header=h; }

        void addDescription(Description);
        void clearDescription() { }

        void display(QString str="");

    private:
        Header                   header;
	Description              description;
        QValueList<Description>  descriptions;
        QValueList<Recipe>       recipes;
	QValueList<ShoppingList> shoppinglists;
};

#endif
