-- File:		drop_all_tables.sql
-- Description:	Drops all ReciPants tables - THIS WILL DELETE ALL OF YOUR
--              ReciPants DATA!
-- Author:		Nick Grossman <nick@pantsblazing.com>
-- Tab stops:	4
-- Version:		1.1


DROP TABLE categories;
DROP TABLE category_entries;
DROP TABLE ingredient_types;
DROP TABLE ingredients;
DROP TABLE instructions;
DROP TABLE recipes;
DROP TABLE units;
DROP TABLE user_access_grants;
DROP TABLE user_permissions;
DROP TABLE user_recipe_bookmarks;
DROP TABLE users;
DROP TABLE login_log;

-- END

-- File:        mythRecipe_drop_all_tables.sql
-- Description:	Drops all mythRecipe tables - THIS WILL DELETE ALL OF YOUR
--              mythRecipe  DATA!
-- Author:
-- Tab stops:	4
-- Version:		1.1



DROP TABLE related_recipes;

-- END

-- File:		tables-mysql.sql
-- Description:	DDL for ReciPants schema, MySQL version
-- Author:		Nick Grossman <nick@pantsblazing.com>
-- Tab stops:	4
-- Version:		1.1.1



------------
-- TABLES --
------------


CREATE TABLE categories (
	category_id 		INT4			NOT NULL AUTO_INCREMENT,
	parent 				INT4 			NOT NULL,
	name 				TEXT			NOT NULL,
	CONSTRAINT 			pk_categories PRIMARY KEY (category_id)
);


CREATE TABLE category_entries (
	recipe_id 			INT4 			NOT NULL,
	category_id			INT4 			NOT NULL,
	CONSTRAINT			pk_category_entries PRIMARY KEY (recipe_id, category_id)
);


CREATE TABLE ingredient_types (
	ingredient_type		INT4			NOT NULL AUTO_INCREMENT,
	name 				VARCHAR(100) 	NOT NULL,
	info 				VARCHAR(100),
	CONSTRAINT 			pk_ingredient_types PRIMARY KEY (ingredient_type)
);


CREATE TABLE ingredients (
	ingredient_id 		INT4			NOT NULL AUTO_INCREMENT,
	recipe_id 			INT4 			NOT NULL,
	display_order 		INT4			,
	ingredient_type 	INT4 			NOT NULL,
	sub_recipe_id 		INT4			,
	name 				VARCHAR(100) 	NOT NULL,
	qty					DECIMAL(10,5)	NOT NULL,
	unit_id 			INT4 			NOT NULL,
	CONSTRAINT 			pk_ingredients PRIMARY KEY (ingredient_id),
	INDEX				indx_ingredients_recipe_id (recipe_id)
);


CREATE TABLE instructions (
	recipe_id 			INT4 			NOT NULL,
	step_num 			INT4 			NOT NULL,
	step_text 			TEXT			,
	INDEX 				indx_instructions_recipe_id (recipe_id)
);


CREATE TABLE recipes (
	recipe_id 			INT4			NOT NULL AUTO_INCREMENT,
	name 				TEXT		 	NOT NULL,
	yield_qty 			INT2			,
	yield_units 		TEXT			,
	source 				TEXT			NOT NULL,
	create_utime 		VARCHAR(20)  	NOT NULL,
	mod_utime 			VARCHAR(20) 	NOT NULL,
	author_user_id 		INT4			NOT NULL,
	CONSTRAINT 			pk_recipes PRIMARY KEY (recipe_id)
);


CREATE TABLE units (
	unit_id 			INT4			NOT NULL AUTO_INCREMENT,
	name 				VARCHAR(100) 	NOT NULL,
	abbreviation 		VARCHAR(10) 	NOT NULL,
	CONSTRAINT 			pk_units PRIMARY KEY (unit_id)
);


CREATE TABLE user_access_grants (
	user_id 			INT4 			NOT NULL,
	permission_id 		INT2 			NOT NULL,
	granter_user_id 	INT4 			NOT NULL,
	grant_utime 		VARCHAR(20) 	NOT NULL,
	INDEX 				indx_user_access_grants_uid (user_id)
);


CREATE TABLE user_permissions (
	permission_id 		INT2 			NOT NULL,
	short_description 	VARCHAR(100) 	NOT NULL,
	long_description 	TEXT			,
	CONSTRAINT 			pk_user_permissions PRIMARY KEY (permission_id)
);


CREATE TABLE user_recipe_bookmarks (
	user_id 			INT4 			NOT NULL,
	recipe_id 			INT4 			NOT NULL,
	url                      	VARCHAR(100),
	CONSTRAINT 			pk_user_recipe_bookmarks PRIMARY KEY (user_id, recipe_id)
);


CREATE TABLE users (
	user_id 			INT4			NOT NULL 		AUTO_INCREMENT,
	user_name 			VARCHAR(30) 	NOT NULL,
	user_name_lower 	VARCHAR(30) 	NOT NULL,
	password 			VARCHAR(40) 	NOT NULL,
	email 				VARCHAR(75) 	NOT NULL,
	url 				VARCHAR(100)	,
	status 				INT2 			DEFAULT 1 		NOT NULL,
	email_private 		INT2			,
	create_utime 		VARCHAR(20)		NOT NULL,
	CONSTRAINT 			pk_users PRIMARY KEY (user_id),
	CONSTRAINT 			indx_unq_users_user_name_lower UNIQUE (user_name_lower)
);


CREATE TABLE login_log (
	user_id				INT4			NOT NULL,
	ip					VARCHAR(16)		,
	hostname			VARCHAR(255)	,
	login_utime			VARCHAR(20)		NOT NULL,
	INDEX 				indx_login_log_user_id (user_id)
);



-----------------
-- CONSTRAINTS --
----------------- 

-- Not available in MySQL. You have been warned!


-- END tables-mysql.sql
-- File:		ref_data.sql
-- Description:	Reference data for recipe manager lookup tables
-- Author:		Nick Grossman <nick@pantsblazing.com>
-- Tab stops:	4
-- Version:		1.1


-- Unit codes
INSERT INTO units (name, abbreviation) VALUES ('Teaspoons', 	'tsp');
INSERT INTO units (name, abbreviation) VALUES ('Tablespoons', 	'tbsp');
INSERT INTO units (name, abbreviation) VALUES ('Cups', 			'cps');
INSERT INTO units (name, abbreviation) VALUES ('Cans', 			'can');
INSERT INTO units (name, abbreviation) VALUES ('Pints', 		'pt');
INSERT INTO units (name, abbreviation) VALUES ('Quarts', 		'qt');
INSERT INTO units (name, abbreviation) VALUES ('Gallons', 		'gal');
INSERT INTO units (name, abbreviation) VALUES ('Mililiters', 	        'ml');
INSERT INTO units (name, abbreviation) VALUES ('Liters', 		'l');
INSERT INTO units (name, abbreviation) VALUES ('Ounces', 		'oz');
INSERT INTO units (name, abbreviation) VALUES ('Fluid ounces', 	     'Fl oz');
INSERT INTO units (name, abbreviation) VALUES ('Grams', 		'g');
INSERT INTO units (name, abbreviation) VALUES ('Kilograms', 	        'kg');
INSERT INTO units (name, abbreviation) VALUES ('Pinches', 		'pinches');
INSERT INTO units (name, abbreviation) VALUES ('Each', 			'ea');
INSERT INTO units (name, abbreviation) VALUES ('Pounds', 		'lb');


-- Ingredient type codes
INSERT INTO ingredient_types (ingredient_type, name) VALUES (1, 'Normal');
INSERT INTO ingredient_types (ingredient_type, name) VALUES (2, 'Sub-heading');
INSERT INTO ingredient_types (ingredient_type, name) VALUES (3, 'Divider');
INSERT INTO ingredient_types (ingredient_type, name) VALUES (4, 'Recipe Link');


-- User permission codes
INSERT INTO user_permissions (permission_id, short_description, long_description) VALUES (0, 'All Access',        'User may do anything possible in the system (root access). If a user has All Access permission, is is not necessary to grant them anything else.');
INSERT INTO user_permissions (permission_id, short_description, long_description) VALUES (1, 'Delete Any Recipe', 'User may delete any recipe in the system.');
INSERT INTO user_permissions (permission_id, short_description, long_description) VALUES (2, 'Edit Any Recipe',   'User may make changes to any recipe in the system.');
INSERT INTO user_permissions (permission_id, short_description, long_description) VALUES (3, 'Edit Any Category', 'User may edit (Create, Rename, Move, Delete) any Category in the system.');


-- Master category
INSERT INTO categories (category_id, parent, name) VALUES ('0', '-1', 'All Categories');
-- Fix category ID (may be set incorrectly when using auto-incrementing types)
UPDATE categories SET category_id = 0 WHERE parent = -1;


-- Administrator user account info
INSERT INTO users (user_id, user_name, user_name_lower, password, email, create_utime) VALUES (1, 'admin', 'admin', 'change_me', 'change_me', '1064093171');
INSERT INTO user_access_grants (user_id, permission_id, granter_user_id, grant_utime) VALUES (1, 0, 0, '1064093171');

-- add for local scripts...
INSERT INTO users (user_id, user_name, user_name_lower, password, email, create_utime) VALUES (2, 'scanner - (www.cooking.com)', 'cooking.com', 'change_me', 'change_me', '1064093171');
INSERT INTO user_access_grants (user_id, permission_id, granter_user_id, grant_utime) VALUES (2, 0, 1, '1064093171');

-- END ref_data.sql
-- File:		ref_data_1.0-1.1_update.sql
-- Description:	Updates reference data from v1.0.x to v1.1
-- Author:		Nick Grossman <nick@pantsblazing.com>
-- Tab stops:	4
-- Version:		1.1


-- Fix category ID (must be 0 and may be set incorrectly when using 
-- auto-incrementing types)
UPDATE categories SET category_id = 0 WHERE parent = -1;

-- This just should have been there.
INSERT INTO units (name, abbreviation) VALUES ('Pounds', 'lb');
-- File:		mythrecipe-tables-mysql.sql
-- Description:	DDL for mythrecipe,ReciPants schema, MySQL version
-- Author:		
-- Tab stops:	4
-- Version:		



------------
-- TABLES --
------------

-- check if recipe is a favourite
ALTER TABLE recipes ADD COLUMN favourite BOOL ;

-- list of related recipes
CREATE TABLE related_recipes (
	recipe_id 			INT4 			NOT NULL,
	related_id			INT4 			NOT NULL,
	CONSTRAINT			pk_category_entries PRIMARY KEY (recipe_id, related_id)
);



-----------------
-- CONSTRAINTS --
----------------- 

-- Not available in MySQL. You have been warned!


-- END tables-mysql.sql
